package com.otk.application.error;

public abstract class AbstractApplicationError extends RuntimeException {

	private static final long serialVersionUID = 1L;

	public AbstractApplicationError() {
		super();
	}

	public AbstractApplicationError(String message, Throwable cause, boolean enableSuppression,
			boolean writableStackTrace) {
		super(message, cause, enableSuppression, writableStackTrace);
	}

	public AbstractApplicationError(String message, Throwable cause) {
		super(message, cause);
	}

	public AbstractApplicationError(String message) {
		super(message);
	}

	public AbstractApplicationError(Throwable cause) {
		super(cause);
	}

	public static void rethrow(Throwable t) {
		throw getToRethrow(t);
	}

	public static RuntimeException getToRethrow(Throwable t) {
		return getToRethrow(null, t, true);
	}

	public static void rethrow(String errorContextDescription, Throwable t) {
		throw getToRethrow(errorContextDescription, t);
	}

	public static RuntimeException getToRethrow(String errorContextDescription, Throwable t) {
		return getToRethrow(errorContextDescription, t, true);
	}

	public static void rethrow(String errorContextDescription, Throwable t, boolean unknownErrorAsUnexpected) {
		throw getToRethrow(errorContextDescription, t, unknownErrorAsUnexpected);
	}

	private static RuntimeException getToRethrow(String errorContextDescription, Throwable t,
			boolean unknownErrorAsUnexpected) {
		if ((t instanceof UnexpectedError) || (!(t instanceof StandardError) && unknownErrorAsUnexpected)) {
			if (errorContextDescription != null) {
				return new UnexpectedError(errorContextDescription + ": " + t.toString(), t);
			} else {
				return new UnexpectedError(t.toString(), t);
			}
		} else {
			if (errorContextDescription != null) {
				return new StandardError(errorContextDescription + ": " + t.toString(), t);
			} else {
				return new StandardError(t.toString(), t);
			}
		}
	}

}
