package com.otk.application.image.filter;

import java.awt.Color;
import java.awt.Point;
import java.awt.image.BufferedImage;

public abstract class AbstractOptimizablePixelFilter extends AbstractPixelFilter {

	protected abstract boolean isPixelAlterable(BufferedImage srcImage, Point pixelPosition, Color pixelColor);

	protected OptimizationCache optimizationCache = new OptimizationCache();

	@Override
	protected void setFieldValue(String paramName, Object filedValue) {
		super.setFieldValue(paramName, filedValue);
		optimizationCache.clear();
	}

	@Override
	final protected Color getNewPixelColor(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		if (isPixelAlterable(srcImage, pixelPosition, pixelColor)) {
			Object optimizationKey = getOptimizationCacheKey(srcImage, pixelPosition, pixelColor);
			Color newPixelColor;
			if (optimizationKey != null) {
				newPixelColor = optimizationCache.get(optimizationKey);
				if (newPixelColor == null) {
					newPixelColor = getNewPixelColorWithoutOptimization(srcImage, pixelPosition, pixelColor);
					optimizationCache.put(optimizationKey, newPixelColor);
				}
			} else {
				newPixelColor = getNewPixelColorWithoutOptimization(srcImage, pixelPosition, pixelColor);
			}
			return newPixelColor;
		} else {
			return pixelColor;
		}
	}

	protected Color getNewPixelColorWithoutOptimization(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		return super.getNewPixelColor(srcImage, pixelPosition, pixelColor);
	}

}