package com.otk.application.image.filter;

import java.awt.Color;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

import com.otk.application.image.FastestPixels;
import com.otk.application.util.ImageUtils;

public abstract class AbstractPixelFilter extends AbstractFilter {
	protected abstract int getNewPixelColorComponentValue(char colorComponent, BufferedImage srcImage,
			Point pixelPosition, Color pixelColor);

	
	@Override
	protected final FilteringContext doExecute(FilteringContext context) {
		return process(context);
	}

	protected final FilteringContext process(FilteringContext context) {
		BufferedImage srcImage = context.getImage();
		onImageProcessingStart(context);
		BufferedImage dstImage = new BufferedImage(srcImage.getWidth(), srcImage.getHeight(),
				ImageUtils.getAdaptedBufferedImageType());
		FastestPixels.transfer(srcImage, dstImage, new FastestPixels.TransferFilter() {
			@Override
			public int get(int x, int y, int rgb, int[] srcPixels) {
				Point pixelPosition = new Point(x, y);
				Color srcPixelColor = new Color(rgb, true);
				Color newPixelColor = getNewPixelColor(srcImage, pixelPosition, srcPixelColor);
				return newPixelColor.getRGB();
			}
		}, getScan(context));
		FilteringContext result = context.withImage(dstImage);
		onImageProcessingEnd(result);
		return result;
	}

	protected Iterable<Point> getScan(FilteringContext inputContext) {
		int width = inputContext.getImage().getWidth();
		int height = inputContext.getImage().getHeight();
		List<Point> scan = new ArrayList<Point>();
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				scan.add(new Point(x, y));
			}
		}
		return scan;
	}

	protected void onImageProcessingStart(FilteringContext inputContext) {
	}

	protected void onImageProcessingEnd(FilteringContext outputContext) {
	}

	protected Color getNewPixelColor(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		int destRed = getNewPixelColorComponentValue('r', srcImage, pixelPosition, pixelColor);
		int destGreen = getNewPixelColorComponentValue('g', srcImage, pixelPosition, pixelColor);
		int destBlue = getNewPixelColorComponentValue('b', srcImage, pixelPosition, pixelColor);
		int destAlpha = getNewPixelColorComponentValue('a', srcImage, pixelPosition, pixelColor);
		destRed = Math.min(255, Math.max(0, destRed));
		destGreen = Math.min(255, Math.max(0, destGreen));
		destBlue = Math.min(255, Math.max(0, destBlue));
		destAlpha = Math.min(255, Math.max(0, destAlpha));
		return new Color(destRed, destGreen, destBlue, destAlpha);
	}

	protected Object getOptimizationCacheKey(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		return null;
	}


}
