package com.otk.application.image.filter;

import java.awt.image.BufferedImage;

import javax.swing.JPanel;
import javax.swing.SpinnerNumberModel;

import com.otk.application.translation.T;
import com.otk.application.util.ImageUtils;
import com.otk.application.util.MathUtils;
import com.otk.application.window.component.crud.ICrudItemControlListener;

public class BoundingBox extends AbstractFilter {

	public int width = 100;
	public int height = 100;
	public boolean scaleToWidth = false;
	public boolean scaleToHeight = false;
	public boolean cutEmptySides = false;

	@Override
	public String getCaption() {
		return T.t("Bounding box");
	}

	@Override
	protected void createFieldControls(JPanel panel, ICrudItemControlListener listener) {
		createSlinnerFieldControl(panel, "width", new SpinnerNumberModel(100, 1, 1000, 1), T.t("width"));
		createSlinnerFieldControl(panel, "height", new SpinnerNumberModel(100, 1, 1000, 1), T.t("height"));
		createCheckBoxFieldControl(panel, "cutEmptySides", T.t("remove empty corners"));
		createCheckBoxFieldControl(panel, "scaleToWidth", T.t("scale to width"));
		createCheckBoxFieldControl(panel, "scaleToHeight", T.t("scale to height"));
	}

	@Override
	protected FilteringContext doExecute(FilteringContext context) {
		BufferedImage image = context.getImage();
		BufferedImage result = null;
		if (scaleToWidth || scaleToHeight) {
			if (scaleToWidth) {
				result = ImageUtils.scaleToWidth(image, width);
				if (!cutEmptySides || (cutEmptySides && (result.getHeight() > height))) {
					int y = MathUtils.round((result.getHeight() - height) / 2.0);
					result = ImageUtils.getBufferedImage(ImageUtils.getOverflowingSubImage(result, 0, y, width, height));
				}
			}
			if (scaleToHeight) {
				result = ImageUtils.scaleToHeight(image, height);
				if (!cutEmptySides || (cutEmptySides && (result.getWidth() > width))) {
					int x = MathUtils.round((result.getWidth() - width) / 2.0);
					result = ImageUtils.getBufferedImage(ImageUtils.getOverflowingSubImage(result, x, 0, width, height));
				}
			}
		} else {
			result = ImageUtils.getBufferedImage(
					ImageUtils.scalePreservingRatio(image, width, height, cutEmptySides, true));
		}
		return context.withImage(result);
	}
}
