package com.otk.application.image.filter;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JOptionPane;

import com.otk.application.error.UnexpectedError;
import com.otk.application.image.camera.Camera;
import com.otk.application.image.camera.FrameFormat;
import com.otk.application.util.MiscUtils;
import com.otk.application.util.Pair;

public class ChainedPixelFilter extends AbstractHeadlessPixelFilter {

	public static void main(String[] args) {
		Camera camera = new Camera();
		camera.setFpsUpdateEnabled(true);
		List<String> deviceNames = camera.listAvailableDevices(null);
		camera.setDeviceFullName(deviceNames.get(deviceNames.size() - 1));
		camera.setVideoFormat(new FrameFormat(1024, 768));
		camera.getVideoComponent().setPreferredSize(new Dimension(400, 300));
		List<AbstractPixelFilter> filters = new ArrayList<AbstractPixelFilter>();
		{
			Saturation filter1 = new Saturation();
			filter1.amount = 100;
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
			filters.add(filter1);
		}

		camera.start();
		camera.setVideoInputCorrectionFilters(
				Collections.singletonList(new MultiFilter(MiscUtils.convertCollection(filters))));
		JOptionPane.showMessageDialog(null, camera.getVideoComponent());
		camera.stop();

	}

	private List<AbstractPixelFilter> filters;

	public ChainedPixelFilter(List<AbstractPixelFilter> filters) {
		this.filters = filters;
	}

	@Override
	protected Object getOptimizationCacheKey(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		Object result = null;
		for (AbstractPixelFilter filter : filters) {
			Object key = filter.getOptimizationCacheKey(srcImage, pixelPosition, pixelColor);
			if (key == null) {
				return null;
			}
			if (result == null) {
				result = key;
			} else {
				result = new Pair<Object, Object>(result, key);
			}
		}
		return result;
	}

	@Override
	protected void onImageProcessingStart(FilteringContext inputContext) {
		for (AbstractPixelFilter filter : filters) {
			filter.onImageProcessingStart(inputContext);
		}
	}

	@Override
	protected void onImageProcessingEnd(FilteringContext outputContext) {
		for (AbstractPixelFilter filter : filters) {
			filter.onImageProcessingEnd(outputContext);
		}
	}

	@Override
	protected int getNewPixelColorComponentValue(char colorComponent, BufferedImage srcImage, Point pixelPosition,
			Color pixelColor) {
		throw new UnexpectedError();
	}

	@Override
	protected Color getNewPixelColor(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		for (AbstractPixelFilter filter : filters) {
			pixelColor = filter.getNewPixelColor(srcImage, pixelPosition, pixelColor);
		}
		return pixelColor;
	}

}
