package com.otk.application.image.filter;

import java.awt.Color;
import java.awt.image.RescaleOp;
import java.util.regex.Pattern;

import javax.swing.JPanel;

import com.otk.application.translation.T;
import com.otk.application.window.component.crud.ICrudItemControlListener;

public class ContrastAndBrightness extends AbstractFilter {

	public int contrastCompensation = 20;
	public int brightnessCompensation = 20;
	public Color contrastPivot = Color.GRAY;

	@Override
	public String getCaption() {
		return T.t("Contrast and brightness");
	}

	@Override
	protected FilteringContext doExecute(FilteringContext context) {
		int MAX_SCALE_FACTOR = 20;
		int MAX_CONTRAST_COMPENSATION = 127;
		float scaleFactor = (float) Math.pow(2.0,
				contrastCompensation * (Math.exp(Math.log(MAX_SCALE_FACTOR) / 2.0) / MAX_CONTRAST_COMPENSATION));
		float rOffset = ((+brightnessCompensation - contrastPivot.getRed()) * scaleFactor) + contrastPivot.getRed();
		float gOffset = ((+brightnessCompensation - contrastPivot.getGreen()) * scaleFactor)
				+ contrastPivot.getGreen();
		float bOffset = ((+brightnessCompensation - contrastPivot.getBlue()) * scaleFactor)
				+ contrastPivot.getBlue();
		RescaleOp rescale = new RescaleOp(new float[] { scaleFactor, scaleFactor, scaleFactor, 1.0f },
				new float[] { rOffset, gOffset, bOffset, 0.0f }, null);
		return context.withImage(rescale.filter(context.getImage(), null));
	}

	@Override
	protected void createFieldControls(JPanel panel, ICrudItemControlListener listener) {
		createSlinnerFieldControl(panel, "brightnessCompensation", -127, 127, T.t("brightness compensation"));
		createSlinnerFieldControl(panel, "contrastCompensation", -127, 127, T.t("contrast compensation"));
		createColorPickerFieldControl(panel, "contrastPivot", T.t("contrast pivot"));
	}

	@Override
	public void readFields(String line) {
		if (!Pattern.compile("contrastPivot=([0-9]+),([0-9]+)([0-9]+)").matcher(line).find()) {
			line = line.replaceAll("contrastPivot=([0-9]+)", "contrastPivot=$1,$1,$1");
		}
		super.readFields(line);
	}

}