package com.otk.application.image.filter;

import java.awt.image.BufferedImage;
import java.util.ArrayList;

import com.otk.application.error.UnexpectedError;
import com.otk.application.util.ImageUtils;

public class FilteringContext implements Cloneable {

	private BufferedImage image;
	private FilteringContext parent;
	private ArrayList<AbstractCompositionFunction> imageCompositions = new ArrayList<AbstractCompositionFunction>();

	public BufferedImage getImage() {
		return image;
	}

	public FilteringContext getParent() {
		return parent;
	}

	public void setParent(FilteringContext parent) {
		this.parent = parent;
	}

	public FilteringContext withNewImageCompositionFunction(AbstractCompositionFunction function) {
		FilteringContext result = clone();
		result.imageCompositions.add(function);
		result.parent = this;
		return result;
	}

	public FilteringContext withoutImageCompositionFunction() {
		FilteringContext result = clone();
		result.imageCompositions.clear();
		result.parent = this;
		return result;
	}

	public FilteringContext withImage(BufferedImage newImage) {
		FilteringContext result = clone(newImage);
		if (image != null) {
			result.image = compose(image, result.image);
		}
		result.parent = this;
		return result;
	}

	private BufferedImage compose(BufferedImage oldImage, BufferedImage newImage) {
		BufferedImage result = newImage;
		for (AbstractCompositionFunction composition : imageCompositions) {
			result = composition.execute(oldImage, result);
		}
		return result;
	}

	@SuppressWarnings("unchecked")
	@Override
	public FilteringContext clone() {
		FilteringContext result;
		try {
			result = (FilteringContext) super.clone();
		} catch (CloneNotSupportedException e) {
			throw new UnexpectedError(e);
		}
		if (image != null) {
			result.image = ImageUtils.copy(image);
		}
		result.imageCompositions = (ArrayList<AbstractCompositionFunction>) imageCompositions.clone();
		result.parent = parent;
		return result;
	}

	@SuppressWarnings("unchecked")
	public FilteringContext clone(BufferedImage newImage) {
		FilteringContext result;
		try {
			result = (FilteringContext) super.clone();
		} catch (CloneNotSupportedException e) {
			throw new UnexpectedError(e);
		}
		result.image = newImage;
		result.imageCompositions = (ArrayList<AbstractCompositionFunction>) imageCompositions.clone();
		result.parent = parent;
		return result;
	}

	public void logInfo(String msg) {
		System.out.println(msg);
	}

	public void logError(String msg) {
		System.err.println(msg);
	}
}