package com.otk.application.image.filter;

import java.util.Arrays;
import java.util.List;

public class MultiFilter extends AbstractHeadlessFilter {

	private List<AbstractFilter> filters;

	public MultiFilter(AbstractFilter... filters) {
		this(Arrays.asList(filters));
	}

	public MultiFilter(List<AbstractFilter> filters) {
		this.filters = filters;
	}

	public List<AbstractFilter> getFilters() {
		return filters;
	}

	@Override
	protected FilteringContext doExecute(FilteringContext context) {
		FilteringContext initialContext = context;
		for (AbstractFilter f : filters) {
			context = f.execute(context);
			FilteringContext forgettingParentContext = context.clone(context.getImage());
			forgettingParentContext.setParent(null);
			context = forgettingParentContext;
		}
		context = context.clone();
		context.setParent(initialContext);
		return context;
	}

}
