package com.otk.application.image.filter;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;
import javax.swing.SpinnerNumberModel;

import com.otk.application.translation.T;
import com.otk.application.util.ImageUtils;
import com.otk.application.util.MathUtils;
import com.otk.application.window.component.crud.ICrudItemControlListener;

public class Rescaling extends AbstractFilter {

	public double widthFactor = 1.0;
	public double heightFactor = 1.0;

	@Override
	public String getCaption() {
		return T.t("Rescaling");
	}

	@Override
	public void createFieldControls(JPanel panel, ICrudItemControlListener listener) {
		createSlinnerFieldControl(panel, "widthFactor", new SpinnerNumberModel(1.0, 0.0, 2.0, 0.005),
				T.t("width factor"));
		createSlinnerFieldControl(panel, "heightFactor", new SpinnerNumberModel(1.0, 0.0, 2.0, 0.005),
				T.t("height factor"));
	}

	@Override
	protected FilteringContext doExecute(FilteringContext context) {
		int newWidth = MathUtils.round(context.getImage().getWidth() * widthFactor);
		int newHeight = MathUtils.round(context.getImage().getHeight() * heightFactor);
		if (newWidth == 0) {
			return context.withImage(ImageUtils.getBufferedImage(ImageUtils.NOT_FOUND_IMAGE));
		}
		if (newHeight == 0) {
			return context.withImage(ImageUtils.getBufferedImage(ImageUtils.NOT_FOUND_IMAGE));
		}
		boolean flipHorizontally = false;
		boolean flipVertically = false;
		if (newWidth < 0) {
			newWidth = -newWidth;
			flipHorizontally = true;
		}
		if (newHeight < 0) {
			flipVertically = true;
			newHeight = -newHeight;
		}
		BufferedImage image = context.getImage();
		BufferedImage newImage = new BufferedImage(newWidth, newHeight, ImageUtils.getAdaptedBufferedImageType());
		Graphics2D g = newImage.createGraphics();
		if (flipHorizontally) {
			g.translate(newWidth, 0);
			g.scale(-1, 1);
		}
		if (flipVertically) {
			g.translate(0, newHeight);
			g.scale(1, -1);
		}
		ImageUtils.drawSmoothlyScaled(g, image, 0, 0, newWidth, newHeight);
		g.dispose();
		return context.withImage(newImage);
	}

}
