package com.otk.application.image.filter;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

import com.otk.application.translation.T;
import com.otk.application.util.ImageUtils;
import com.otk.application.util.MathUtils;
import com.otk.application.window.component.crud.ICrudItemControlListener;

public class Rotation extends AbstractFilter {

	public int angleDegrees = 0;

	@Override
	public String getCaption() {
		return T.t("Rotation");
	}

	@Override
	public void createFieldControls(JPanel panel, ICrudItemControlListener listener) {
		createSlinnerFieldControl(panel, "angleDegrees", -180, 180, T.t("angle (degrees)"));
	}

	@Override
	protected FilteringContext doExecute(FilteringContext context) {
		if (angleDegrees == 0) {
			return context;
		}
		BufferedImage image = context.getImage();

		Dimension size = new Dimension(image.getWidth(), image.getHeight());
		Dimension newSize = getBoxBounds(0, 0, size.width, size.height, angleDegrees).getSize();

		BufferedImage newImage = new BufferedImage(newSize.width, newSize.height,
				ImageUtils.getAdaptedBufferedImageType());
		Graphics2D g = newImage.createGraphics();
		if ((angleDegrees % 90) != 0) {
			ImageUtils.setSmoothScalingRenderingHints(g);
		}
		AffineTransform at = getRotationTransform(size.width, size.height, angleDegrees);
		g.setTransform(at);
		g.drawImage(image, 0, 0, null);
		g.dispose();
		return context.withImage(newImage);
	}

	public static AffineTransform getRotationTransform(int w, int h, int angleDegrees) {
		Dimension newSize = getBoxBounds(0, 0, w, h, angleDegrees).getSize();
		double radians = -Math.toRadians(angleDegrees);
		AffineTransform at = new AffineTransform();
		at.translate((newSize.width - w) / 2, (newSize.height - h) / 2);
		int cWidth = MathUtils.round(w / 2.0);
		int cHeight = MathUtils.round(h / 2.0);
		at.rotate(radians, cWidth, cHeight);
		return at;
	}

	public static Rectangle getBoxBounds(int x, int y, int w, int h, int angleDegrees) {
		double radians = -Math.toRadians(angleDegrees);
		double sin, cos;

		sin = Math.abs(Math.sin(radians));
		cos = Math.abs(Math.cos(radians));
		int newWidth = MathUtils.round(Math.floor(w * cos + h * sin));

		sin = Math.abs(Math.sin(radians));
		cos = Math.abs(Math.cos(radians));
		int newHeight = MathUtils.round(Math.floor(h * cos + w * sin));

		x += MathUtils.round((w - newWidth) / 2.0);
		y += MathUtils.round((h - newHeight) / 2.0);
		return new Rectangle(x, y, newWidth, newHeight);
	}
}
