package com.otk.application.image.filter;

import java.awt.Color;
import java.awt.Point;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;
import com.otk.application.error.UnexpectedError;
import com.otk.application.translation.T;
import com.otk.application.util.MathUtils;
import com.otk.application.window.component.crud.ICrudItemControlListener;

public class Saturation extends AbstractOptimizablePixelFilter {

	public int amount = 50;

	@Override
	public String getCaption() {
		return T.t("Saturation");
	}

	@Override

	public void createFieldControls(JPanel panel, final ICrudItemControlListener listener) {
		createSlinnerFieldControl(panel, "amount", 0, 100, T.t("amount"));
	}

	@Override

	protected int getNewPixelColorComponentValue(char c, BufferedImage im, Point p, Color pixelColor) {
		int currentComponent, otherComponent1, otherComponent2;
		if (c == 'r') {
			currentComponent = pixelColor.getRed();
			otherComponent1 = pixelColor.getGreen();
			otherComponent2 = pixelColor.getBlue();
		} else if (c == 'g') {
			currentComponent = pixelColor.getGreen();
			otherComponent1 = pixelColor.getRed();
			otherComponent2 = pixelColor.getBlue();
		} else if (c == 'b') {
			currentComponent = pixelColor.getBlue();
			otherComponent1 = pixelColor.getGreen();
			otherComponent2 = pixelColor.getRed();
		} else if (c == 'a') {
			return pixelColor.getAlpha();
		} else {
			throw new UnexpectedError();
		}
		double diffFromGray = currentComponent - (currentComponent + otherComponent1 + otherComponent2) / 3.0;
		double diffFactor = 2 * amount / 100.0;
		return MathUtils.round(currentComponent - diffFromGray + diffFactor * diffFromGray);
	}

	@Override
	protected boolean isPixelAlterable(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		return true;
	}

	@Override
	protected Object getOptimizationCacheKey(BufferedImage srcImage, Point pixelPosition, Color pixelColor) {
		return pixelColor.getRGB();
	}

}
