package com.otk.application.image.filter;

import java.awt.Color;
import java.awt.Point;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

import com.otk.application.error.UnexpectedError;
import com.otk.application.translation.T;
import com.otk.application.window.component.crud.ICrudItemControlListener;

public class SelectChannel extends AbstractFilter {

	public boolean redChannelFiltered = true;
	public boolean greenChannelFiltered = false;
	public boolean blueChannelFiltered = false;
	public boolean alphaChannelFiltered = false;

	@Override
	public String getCaption() {
		return T.t("Select channel (the following filters will only affect the specified color components)");
	}

	@Override
	protected FilteringContext doExecute(FilteringContext context) {
		context = context.withNewImageCompositionFunction(new AbstractCompositionFunction() {

			@Override
			protected int getNewPixelColorComponentValue(BufferedImage oldImage, BufferedImage newImage,
					char componentName, Point pixelPosition, Color oldColor, Color newColor) {
				if (isChannelFiltered(componentName)) {
					return getColorComponentValue(oldColor, componentName);
				} else {
					return getColorComponentValue(newColor, componentName);
				}
			}
		});
		return context;
	}

	private boolean isChannelFiltered(char componentName) {
		if (componentName == 'r') {
			return redChannelFiltered;
		}
		if (componentName == 'g') {
			return greenChannelFiltered;
		}
		if (componentName == 'b') {
			return blueChannelFiltered;
		}
		if (componentName == 'a') {
			return alphaChannelFiltered;
		}
		throw new UnexpectedError();
	}

	@Override
	protected void createFieldControls(JPanel panel, ICrudItemControlListener listener) {
		createCheckBoxFieldControl(panel, "redChannelFiltered", T.t("exclude red channel"));
		createCheckBoxFieldControl(panel, "greenChannelFiltered", T.t("exclude green channel"));
		createCheckBoxFieldControl(panel, "blueChannelFiltered", T.t("exclude blue channel"));
		createCheckBoxFieldControl(panel, "alphaChannelFiltered", T.t("exclude alpha channel"));
	}

	@Override
	public Category getCategory() {
		return Category.FILTER_COMPOSER;
	}

}