package com.otk.application.image.printer;

import java.awt.Dimension;
import java.awt.image.BufferedImage;

import com.otk.application.error.UnexpectedError;
import com.otk.application.util.ImageUtils;

public enum PageOrientation {
	Landscape, Portrait;

	public static Dimension flip(Dimension size) {
		return new Dimension(size.height, size.width);
	}

	public static BufferedImage flip(BufferedImage image) {
		PageOrientation sourcePageOrientation = get(image);
		PageOrientation targetPageOrientation = PageOrientation.getOther(sourcePageOrientation);
		if (targetPageOrientation == Landscape) {
			return ImageUtils.rotate(image, -90);
		} else if (targetPageOrientation == Portrait) {
			return ImageUtils.rotate(image, 90);
		} else {
			throw new UnexpectedError();
		}
	}

	public static PageOrientation getOther(PageOrientation pageOrientation) {
		if (pageOrientation == Portrait) {
			return Landscape;
		} else if (pageOrientation == Landscape) {
			return Portrait;
		} else {
			throw new UnexpectedError();
		}
	}

	public static PageOrientation get(BufferedImage image) {
		return get(new Dimension(image.getWidth(), image.getHeight()));
	}

	public static BufferedImage set(BufferedImage image, PageOrientation pageOrientation) {
		if (pageOrientation == get(image)) {
			return image;
		}
		return flip(image);
	}

	public static PageOrientation get(Dimension size) {
		if (size.width > size.height) {
			return PageOrientation.Landscape;
		} else {
			return PageOrientation.Portrait;
		}
	}

	public static Dimension set(Dimension size, PageOrientation pageOrientation) {
		if (pageOrientation == get(size)) {
			return size;
		}
		return flip(size);
	}
}
