package com.otk.application.image.printer;

import java.util.ArrayList;
import java.util.List;

import javax.print.PrintService;
import javax.print.event.PrintJobEvent;
import javax.print.event.PrintJobListener;

import com.otk.application.error.StandardError;
import com.otk.application.util.MiscUtils;
import com.otk.application.util.TextUtils;

public abstract class PrintingStrategy {

	private List<String> report = new ArrayList<String>();
	private Throwable lastError;
	private PrintingManager printingManager;

	public abstract PrintService getTestFakePrintService();

	public abstract void handleEnvironmentSetupEvent();

	public abstract void handleEnvironmentCleanupEvent();

	public abstract void handlePrintjobStartEvent() throws StandardError;

	public abstract void handlePrintJobStopEvent() throws StandardError;

	public abstract void handlePrintJobNoMoreEvents(PrintJobListener printJobListener,
			PrintJobEvent paramPrintJobEvent);

	public abstract void handlePrintingInitializationEvent();

	public abstract void handlePrintingTerminationEvent();

	public abstract void handlePrintJobErrorEvent(Throwable t);

	public PrintingStrategy(PrintingManager printingManager) {
		super();
		this.printingManager = printingManager;
	}

	protected void scheduleNewPrintingAttempt() {
		printingManager.logInfo("Scheduling another printing attempt");
		getManager().setRetrying(true);
	}

	protected boolean isPrintingAttemptScheduled() {
		return getManager().isRetrying();
	}

	public void resetReport() {
		report.clear();
	}

	public void reportEvent(String evtDesc) {
		report.add(0, "- " + TextUtils.getDefaultDateFormat().format(MiscUtils.now()) + " " + evtDesc);
	}

	protected String enrichErrorMessage(String msg) {
		String result = "Printing error: " + msg.toString();
		if (report.size() > 0) {
			result = result + "\n>>> Events:\n" + TextUtils.stringJoin(report, "\n") + "\n<<<";
		}
		return result;
	}

	protected PrintService getPrintService() {
		return getManager().getPrintService();
	}

	protected PrintingManager getManager() {
		return printingManager;
	}

	public Throwable getLastError() {
		return lastError;
	}

	protected void setLastError(Throwable error) {
		this.lastError = error;
		if (error != null) {
			printingManager.logError(error.toString());
		}
	}

	public boolean triggeredRecentWakeUp() {
		return false;
	}

}