package com.otk.application.image.video;

import java.awt.Image;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class SimpleVideoSequence implements IVideoSequence {

	private List<Image> images = new ArrayList<Image>();
	private float fps = 0f;
	private boolean estimatingFps = false;
	private long fpsEstimationStartTime;
	private boolean recording = false;

	public SimpleVideoSequence() {
	}

	public SimpleVideoSequence(float fps, Image... images) {
		this.fps = fps;
		this.images.addAll(Arrays.asList(images));
	}

	@Override
	public float getFps() {
		return fps;
	}

	public void setFps(float fps) {
		this.fps = fps;
	}

	public void beginRecording(boolean estimateFps) {
		images.clear();
		if (estimateFps) {
			fpsEstimationStartTime = System.currentTimeMillis();
			estimatingFps = true;
		}
		recording = true;
	}

	public void endRecording() {
		recording = false;
		if (estimatingFps) {
			estimatingFps = false;
			long elapsedTime = System.currentTimeMillis() - fpsEstimationStartTime;
			fps = (1000f * images.size()) / elapsedTime;
		}
	}

	public boolean isRecording() {
		return recording;
	}

	public void recordFrame(Image image) {
		images.add(image);
	}

	public int getFrameCount() {
		return images.size();
	}

	@Override
	public Iterator<Image> createFrameIterator() {
		return new Iterator<Image>() {

			int cursor = 0;

			@Override
			public Image next() {
				Image result = images.get(cursor);
				cursor++;
				return result;
			}

			@Override
			public boolean hasNext() {
				return cursor < images.size();
			}
		};
	}

	public Iterator<Image> getReturningFrameIterator() {
		return new Iterator<Image>() {

			int cursor = 0;
			boolean returning = false;

			@Override
			public Image next() {
				Image result = images.get(cursor);
				if (cursor == (images.size() - 1)) {
					returning = true;
				}
				if (!returning) {
					cursor++;
				} else {
					cursor--;
				}
				return result;
			}

			@Override
			public boolean hasNext() {
				return !returning || (cursor >= 0);
			}

		};
	}

	@Override
	public void disposeFrameIterator(Iterator<Image> frameIterator) {
	}

}
