package com.otk.application.service;

public abstract class AbstractHotConfigurationManagement <T>{
	private T currentDriver;

	protected abstract T getExpectedDriverName();

	protected abstract boolean sameDriver(T driver1, T driver2);

	protected abstract void installExpectedDriver();

	protected abstract void uninstallCurrentDriver();

	protected abstract boolean isDeviceActive();

	protected abstract void activateDevice();

	protected abstract void deactivateDevice();

	public synchronized void manage() {
		T expectedDriver = getExpectedDriverName();
		if ((expectedDriver != null) && (currentDriver != null)) {
			if (sameDriver(expectedDriver, currentDriver)) {
				return;
			}
		}
		boolean wasDriverActive = false;
		if (currentDriver != null) {
			if (isDeviceActive()) {
				wasDriverActive = true;
				deactivateDevice();
			}
			uninstallCurrentDriver();
		}
		currentDriver = expectedDriver;
		if (currentDriver != null) {
			installExpectedDriver();
			if (wasDriverActive) {
				activateDevice();
			}
		}
	}

}
