package com.otk.application.translation;

import java.util.Collections;
import java.util.Enumeration;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Set;

import com.otk.application.AbstractApplication;
import com.otk.application.Configuration.GlobalConfiguration;
import com.otk.application.translation.T.ITranslationHandler;

public class TranslationHandler implements ITranslationHandler {

	private static final ResourceBundle EMPTY_BUNDLE = new ResourceBundle() {

		@Override
		protected Object handleGetObject(String key) {
			return key;
		}

		@Override
		public Enumeration<String> getKeys() {
			return Collections.emptyEnumeration();
		}
	};

	private ResourceBundle translations;
	private Set<String> overridableTranslationKeys;
	private AbstractApplication application;

	public TranslationHandler(AbstractApplication application) {
		this.application = application;
	}

	public AbstractApplication getApplication() {
		return application;
	}

	public Locale getBaseLocale() {
		return Locale.ENGLISH;
	}

	public Locale[] getSupportedLocales() {
		return new Locale[] { getBaseLocale() };
	}

	public void loadTranslations() {
		if (getTranslationResourceBaseName() == null) {
			translations = EMPTY_BUNDLE;
		} else {
			translations = ResourceBundle.getBundle(getTranslationResourceBaseName(), Locale.getDefault());
		}
		if (getOverridableTranslationResourceBaseName() == null) {
			overridableTranslationKeys = Collections.emptySet();
		} else {
			overridableTranslationKeys = ResourceBundle
					.getBundle(getOverridableTranslationResourceBaseName(), Locale.getDefault()).keySet();
		}
	}

	protected String getTranslationResourceBaseName() {
		return null;
	}

	protected String getOverridableTranslationResourceBaseName() {
		return null;
	}

	public ResourceBundle getTranslations() {
		return translations;
	}

	public Set<String> getOverridableTranslationKeys() {
		return overridableTranslationKeys;
	}

	protected String getOverride(String s) {
		Properties overriddenMessages = getOverridenMessages();
		if (overriddenMessages == null) {
			return s;
		}
		return overriddenMessages.getProperty(s);
	}

	protected Properties getOverridenMessages() {
		return null;
	}

	public Locale getMostSuitedLocale() {
		for (Locale locale : getSupportedLocales()) {
			if (locale.getLanguage().equals(Locale.getDefault().getLanguage())) {
				return locale;
			}
		}
		return getSupportedLocales()[0];
	}

	@Override
	public String translate(String s) {
		try {
			return translations.getString(s);
		} catch (MissingResourceException e) {
			if (getApplication().getConfiguration().getCategory(GlobalConfiguration.class).logVerbose) {
				getApplication().getLog().error(e);
			}
			return s;
		}
	}

	@Override
	public String translateOverridable(String s) {
		s = translate(s);
		String override = getOverride(s);
		if (override != null) {
			return override;
		} else {
			return s;
		}
	}

}