package com.otk.application.util;

import com.otk.application.error.UnexpectedError;

public abstract class AbstractLazilyInitialized {

	private boolean initialized = false;
	private boolean initializationFailed = false;
	private boolean shutDownFailed = false;
	private Object stateChangeMutex = new Object();

	protected abstract boolean initialize() throws Throwable;

	protected abstract boolean shutDown() throws Throwable;

	public boolean isInitialized() {
		return initialized;
	}

	public boolean hasInitializationFailed() {
		return initializationFailed;
	}

	public boolean ensureIsInitialized() {
		synchronized (stateChangeMutex) {
			if (!initialized) {
				initialized = true;
				initializationFailed = true;
				try {
					initializationFailed = !initialize();
				} catch (Throwable t) {					
					throw new UnexpectedError(t);
				}
			}
			return !initializationFailed;
		}
	}

	public boolean ensureIsShutDown() {
		synchronized (stateChangeMutex) {
			if (initialized) {
				initialized = false;
				shutDownFailed = true;
				try {
					shutDownFailed = !shutDown();
				} catch (Throwable t) {
					throw new UnexpectedError(t);
				}
			}
			return !shutDownFailed;
		}
	}

}
