package com.otk.application.util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.AWTEventListener;
import java.awt.event.AWTEventListenerProxy;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.Proxy;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Queue;
import java.util.StringJoiner;
import java.util.Vector;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.nimbus.NimbusLookAndFeel;

import com.otk.application.Configuration;
import com.otk.application.error.AbstractApplicationError;
import com.otk.application.error.Assertion;
import com.otk.application.error.ErrorManager;
import com.otk.application.error.StandardError;
import com.otk.application.error.UnexpectedError;
import com.otk.application.translation.T;

public class MiscUtils {

	public static JPanel flowInLayout(Component c, int gridBagConstraintsAnchor) {
		return flowInLayout(Collections.singletonList(c), gridBagConstraintsAnchor);
	}

	public static JPanel flowInLayout(List<Component> components, int gridBagConstraintsAnchor) {
		JPanel result = new JPanel();
		result.setOpaque(false);
		result.setLayout(new GridBagLayout());
		GridBagConstraints contraints = new GridBagConstraints();
		contraints.gridx = 0;
		contraints.gridy = 0;
		contraints.weightx = 1;
		contraints.weighty = 1;
		contraints.anchor = gridBagConstraintsAnchor;
		for (Component c : components) {
			result.add(c, contraints);
		}
		return result;
	}

	public static String getPrintedStackTrace(Throwable t) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		t.printStackTrace(new PrintStream(out));
		return out.toString();
	}

	public static <C> C getLast(List<C> list) {
		return list.get(list.size() - 1);
	}

	public static Properties toSortedProperties(String text) {
		Properties result = getSortedProperties();
		try {
			result.load(new ByteArrayInputStream(text.getBytes()));
		} catch (IOException e) {
			throw new UnexpectedError(e);
		}
		return result;
	}

	public static Properties getSortedProperties() {
		return new Properties() {
			private static final long serialVersionUID = 1L;

			@Override
			public synchronized Enumeration<Object> keys() {
				Enumeration<Object> keysEnum = super.keys();
				Vector<Object> keyList = new Vector<Object>();
				while (keysEnum.hasMoreElements()) {
					keyList.add(keysEnum.nextElement().toString());
				}
				Collections.sort(keyList, new Comparator<Object>() {
					@Override
					public int compare(Object o1, Object o2) {
						return o1.toString().compareTo(o2.toString());
					}
				});
				return keyList.elements();
			}
		};
	}

	public static OutputStream getNullOutputStream() {
		return new OutputStream() {
			@Override
			public void write(int b) throws IOException {
			}
		};
	}

	public static OutputStream unifyOutputStreams(final OutputStream... outputStreams) {
		return new OutputStream() {

			@Override
			public void write(int b) throws IOException {
				for (OutputStream out : outputStreams) {
					out.write(b);
				}
			}

			@Override
			public void write(byte[] b) throws IOException {
				for (OutputStream out : outputStreams) {
					out.write(b);
				}
			}

			@Override
			public void write(byte[] b, int off, int len) throws IOException {
				for (OutputStream out : outputStreams) {
					out.write(b, off, len);
				}
			}

			@Override
			public void flush() throws IOException {
				for (OutputStream out : outputStreams) {
					out.flush();
				}
			}

			@Override
			public void close() throws IOException {
				for (OutputStream out : outputStreams) {
					out.close();
				}
			}
		};
	}

	public static Thread redirectStream(final InputStream src, final OutputStream dst, String reason) {
		Thread thread = new Thread(MiscUtils.formatThreadName(MiscUtils.class, "Stream Redirector (" + reason + ")")) {
			public void run() {
				try {
					while (true) {
						if (src.available() > 0) {
							int b = src.read();
							if (b == -1) {
								break;
							}
							dst.write(b);
						} else {
							if (isInterrupted()) {
								break;
							} else {
								MiscUtils.relieveCPU();
							}
						}
					}
				} catch (IOException e) {
					if (isStreamClosedException(e)) {
						return;
					}
					throw new UnexpectedError(e);
				}
			}
		};
		thread.setDaemon(true);
		thread.setPriority(Thread.MIN_PRIORITY);
		thread.start();
		return thread;
	}

	public static String formatThreadName(Class<?> class1, String role) {
		String result = class1.getSimpleName();
		long threadNumber = Counter.next(result);
		result += " " + role + " " + threadNumber;
		return result;
	}

	public static void showBusyCursor(Component c, boolean b) {
		Window window = getWindow(c);
		if (window == null) {
			return;
		}
		if (b) {
			window.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		} else {
			window.setCursor(Cursor.getDefaultCursor());
		}
	}

	public static void showBusyCusrorWhile(final Component c, final Runnable runnable) {
		Window window = getWindow(c);
		if (window == null) {
			runnable.run();
		} else {
			showBusyCursor(c, true);
			try {
				runnable.run();
			} finally {
				showBusyCursor(c, false);
			}
		}
	}

	public static Window getWindow(Component c) {
		if (c instanceof Window) {
			return (Window) c;
		} else {
			return SwingUtilities.getWindowAncestor(c);
		}

	}

	public static void showBusyDialogWhile(final Component ownerComponent, final Runnable runnable, String title) {
		final JLabel busyLabel = new JLabel();
		busyLabel.setHorizontalAlignment(SwingConstants.CENTER);
		busyLabel.setText(T.t("Please wait..."));
		busyLabel.setVerticalTextPosition(SwingConstants.TOP);
		busyLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		final JDialog dialog = new JDialog(
				(ownerComponent == null) ? null : SwingUtilities.getWindowAncestor(ownerComponent));
		dialog.setTitle(title);
		dialog.getContentPane().setLayout(new BorderLayout());
		dialog.getContentPane().add(busyLabel);
		dialog.setLocationRelativeTo(null);
		dialog.pack();
		final Throwable[] error = new Throwable[1];
		final Thread thread = new Thread(title) {
			{
				setDaemon(true);
			}

			@Override
			public void run() {
				try {
					runnable.run();
				} catch (Throwable t) {
					error[0] = t;
				} finally {
					dialog.dispose();
				}
			}
		};
		dialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				thread.interrupt();
			}
		});
		thread.start();
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			throw new UnexpectedError(e);
		}
		dialog.setVisible(true);
	}

	public static String inputPassword(String title, String message, Component parentComponent) {
		JPanel panel = new JPanel();

		JLabel messageLabel = new JLabel(message);
		panel.add(messageLabel);

		JPasswordField passwordField = new JPasswordField(20);
		passwordField.setColumns(20);
		panel.add(passwordField);

		int status = JOptionPane.showConfirmDialog(parentComponent, panel, title, JOptionPane.OK_CANCEL_OPTION);
		if (status != JOptionPane.OK_OPTION) {
			return null;
		}
		return new String(passwordField.getPassword());
	}

	public static Window getCurrentWindow() {
		return KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusedWindow();
	}

	public static void relieveCPU() {
		Integer sleepDurationMilliseconds = Integer
				.valueOf(System.getProperty(MiscUtils.class.getName() + ".realTimeAtomMilliseconds", "10"));
		sleepSafely(sleepDurationMilliseconds);
	}

	public static void sleepSafely(long durationMilliseconds) {
		try {
			Thread.sleep(durationMilliseconds);
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
		}
	}

	public static Date now() {
		return new Date();
	}

	public static void askAndPrintAsynchronously(final Image image, Component parent) {
		PrinterJob printJob = PrinterJob.getPrinterJob();
		printJob.setPrintable(new Printable() {
			@Override
			public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
				if (pageIndex == 0) {
					ImageUtils.drawPreservingRatio((Graphics2D) graphics, image, pageFormat.getImageableX(),
							pageFormat.getImageableY(), pageFormat.getImageableWidth(), pageFormat.getImageableHeight(),
							true);
					return PAGE_EXISTS;
				} else {
					return NO_SUCH_PAGE;
				}
			}
		});
		if (!printJob.printDialog()) {
			return;
		}
		Thread thread = new Thread(MiscUtils.formatThreadName(MiscUtils.class, "Asynchronous Printing Handler")) {
			{
				setDaemon(true);
			}

			@Override
			public void run() {
				try {
					printJob.print();
				} catch (PrinterException e) {
					System.out.println(e);
				}
			}
		};
		thread.setPriority(Thread.MIN_PRIORITY);
		thread.start();
		JOptionPane.showMessageDialog(parent, T.t("Printing request sent"));
	}

	public static void loadFieldValuesLine(String line, String[] fieldNames, Object obj) {
		for (String fieldName : fieldNames) {
			Pattern pattern = Pattern.compile("(?:" + fieldName + "=([^;]*)" + ")");
			Matcher matcher = pattern.matcher(line);
			if (!matcher.find()) {
				continue;
			}
			try {
				Field field = obj.getClass().getField(fieldName);
				String stringValue = matcher.group(1);
				stringValue = stringValue.replace(";;", ";");
				stringValue = TextUtils.unescapeNewLine(stringValue);
				Object value = Configuration.deserialize(stringValue, field);
				setFieldValue(obj, fieldName, value);
			} catch (Exception e) {
				throw new StandardError(obj.getClass().getName() + " deserialization error from string '" + line + "'"
						+ ": Field '" + fieldName + "': " + e.toString(), e);
			}
		}
	}

	public static String saveFieldValuesLine(String[] fieldNames, Object obj) {
		StringBuilder result = new StringBuilder();
		for (String fieldName : fieldNames) {
			if (Arrays.asList(fieldNames).indexOf(fieldName) > 0) {
				result.append("; ");
			}
			Object fieldValue = MiscUtils.getFieldValue(obj, fieldName);
			String filedValueString = Configuration.serialize(fieldValue);
			filedValueString = TextUtils.escapeNewLines(filedValueString);
			filedValueString = filedValueString.replace(";", ";;");
			result.append(fieldName + "=" + filedValueString);
		}
		return result.toString();
	}

	public static void setFieldValue(final Object obj, final String fieldName, Object value) {
		Assertion.check(new Assertion.Predicate() {
			@Override
			protected boolean isTrue() {
				return Arrays.asList(getPublicFieldNames(obj.getClass())).contains(fieldName);
			}
		});
		Field field;
		try {
			field = obj.getClass().getField(fieldName);
			field.set(obj, value);
		} catch (Exception e) {
			throw new UnexpectedError(e);
		}
	}

	public static Object getFieldValue(final Object obj, final String fieldName) {
		return getFieldValue(obj.getClass(), fieldName, obj);

	}

	public static Object getFieldValue(final Class<? extends Object> objClass, final String fieldName, Object obj) {
		Assertion.check(new Assertion.Predicate() {
			@Override
			protected boolean isTrue() {
				return Arrays.asList(getPublicFieldNames(objClass)).contains(fieldName);
			}
		});
		try {
			return objClass.getField(fieldName).get(obj);
		} catch (Exception e) {
			throw new UnexpectedError(e);
		}
	}

	public static String[] getPublicFieldNames(Class<? extends Object> class1) {
		List<String> result = new ArrayList<String>();
		for (Field field : class1.getFields()) {
			result.add(field.getName());
		}
		return result.toArray(new String[result.size()]);
	}

	public static Object getInaccesibleFieldValue(Class<?> clazz, Object obj, String fieldName) {
		try {
			Field field = clazz.getDeclaredField(fieldName);
			field.setAccessible(true);
			return field.get(obj);
		} catch (Exception e) {
			throw new UnexpectedError(e);
		}
	}

	public static void setInaccesibleFieldValue(Class<?> clazz, Object obj, String fieldName, Object value) {
		try {
			Field field = clazz.getDeclaredField(fieldName);
			field.setAccessible(true);
			field.set(obj, value);
		} catch (Exception e) {
			throw new UnexpectedError(e);
		}
	}

	public static Object invokeInaccessibleMethod(Class<?> clazz, Object obj, String methodName,
			Class<?>[] parameterTypes, Object[] args) {
		try {
			Method method = clazz.getDeclaredMethod(methodName, parameterTypes);
			method.setAccessible(true);
			return method.invoke(obj, args);
		} catch (Exception e) {
			throw new UnexpectedError(e);
		}
	}

	public static Object getInaccesibleFieldValue(Object obj, String fieldName) {
		return getInaccesibleFieldValue(obj.getClass(), obj, fieldName);
	}

	public static void setInaccesibleFieldValue(Object obj, String fieldName, Object value) {
		setInaccesibleFieldValue(obj.getClass(), obj, fieldName, value);
	}

	public static Object invokeInaccessibleMethod(Object obj, String methodName, Class<?>[] parameterTypes,
			Object[] args) {
		return invokeInaccessibleMethod(obj.getClass(), obj, methodName, parameterTypes, args);
	}

	public static int secondsElapsedSince(Date date) {
		Date now = now();
		long diff = now.getTime() - date.getTime();
		return MathUtils.round((double) diff / (1000));
	}

	public static <BASE, C extends BASE> List<BASE> convertCollection(Collection<C> ts) {
		List<BASE> result = new ArrayList<BASE>();
		for (C t : ts) {
			result.add((BASE) t);
		}
		return result;
	}

	@SuppressWarnings("unchecked")
	public static <BASE, C extends BASE> List<C> convertCollectionUnsafely(Collection<BASE> bs) {
		List<C> result = new ArrayList<C>();
		for (BASE b : bs) {
			result.add((C) b);
		}
		return result;
	}

	public static void fillBoxLayout(Component c) {
		c.setMaximumSize(new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE));
	}

	public static Rectangle getSmallestBoxWithRatio(Rectangle rectangle, double ratioWidth, double ratioHeight) {
		Rectangle result = new Rectangle(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
		double ratio = ratioWidth / ratioHeight;
		double resultRatio = (double) result.width / (double) result.height;
		if (ratio > resultRatio) {
			int newWidth = MathUtils.round(result.height * ratio);
			result.x -= MathUtils.round((newWidth - result.width) / 2.0);
			result.width = newWidth;
		} else {
			int newHeight = MathUtils.round(result.width / ratio);
			result.y -= MathUtils.round((newHeight - result.height) / 2.0);
			result.height = newHeight;
		}
		return result;
	}

	public static void showLargeMessageDialog(Component parent, String title, Image iconImage, String text,
			int jOptionPaneMessageType) {
		JTextArea textArea = new JTextArea(text);
		textArea.setEditable(false);
		textArea.setMargin(new Insets(5, 5, 5, 5));
		JScrollPane scrollTextArea = new JScrollPane(textArea);
		JOptionPane optionPane = new JOptionPane(scrollTextArea, jOptionPaneMessageType, JOptionPane.DEFAULT_OPTION,
				null, null, null);
		JDialog dialog = optionPane.createDialog(parent, title);
		dialog.setIconImage(iconImage);
		dialog.setBounds(MiscUtils.getWindowAverageBounds());
		dialog.setModal(true);
		dialog.setVisible(true);
		dialog.dispose();
	}

	public static void changeLayoutContraints(Component c, Object newConstraints) {
		Container container = c.getParent();
		int index = indexOf(container, c);
		container.remove(c);
		container.add(c, newConstraints, index);
	}

	public static int indexOf(Container container, Component component) {
		for (int i = 0; i < container.getComponentCount(); i++) {
			Component candidate = container.getComponent(i);
			if (component == candidate) {
				return i;
			}
		}
		return -1;
	}

	public static int removeAWTEventListener(AWTEventListener listener) {
		final List<AWTEventListener> listenersToRemove = new ArrayList<AWTEventListener>();
		for (AWTEventListener l : Toolkit.getDefaultToolkit().getAWTEventListeners()) {
			if (l == listener) {
				listenersToRemove.add(l);
			} else if (l instanceof AWTEventListenerProxy) {
				final AWTEventListenerProxy proxyListener = (AWTEventListenerProxy) l;
				if (proxyListener.getListener() == listener) {
					listenersToRemove.add(proxyListener);
				}
			}
		}
		for (AWTEventListener l : listenersToRemove) {
			Toolkit.getDefaultToolkit().removeAWTEventListener(l);
		}
		return listenersToRemove.size();
	}

	public static void disableComponentTree(JComponent c, final boolean revert) {
		String CONTAINER_LISTENER_KEY = MiscUtils.class.getName() + ".disableComponentTree.CONTAINER_LISTENER_KEY";
		String LAST_STATE_KEY = MiscUtils.class.getName() + ".disableComponentTree.LAST_STATE_KEY";
		Boolean lastState = (Boolean) c.getClientProperty(LAST_STATE_KEY);
		if (revert) {
			if (lastState == null) {
				return;
			}
			if (lastState) {
				c.setEnabled(true);
			}
			c.putClientProperty(LAST_STATE_KEY, null);
			ContainerListener containerListener = (ContainerListener) c.getClientProperty(CONTAINER_LISTENER_KEY);
			c.removeContainerListener(containerListener);
		} else {
			if (lastState != null) {
				return;
			}
			c.putClientProperty(LAST_STATE_KEY, c.isEnabled());
			c.setEnabled(false);
			ContainerListener containerListener = new ContainerListener() {

				@Override
				public void componentRemoved(ContainerEvent e) {
				}

				@Override
				public void componentAdded(ContainerEvent e) {
					Component child = e.getChild();
					if (!(child instanceof JComponent)) {
						return;
					}
					disableComponentTree((JComponent) child, revert);
				}
			};
			c.addContainerListener(containerListener);
			c.putClientProperty(CONTAINER_LISTENER_KEY, containerListener);
		}
		for (Component child : c.getComponents()) {
			if (!(child instanceof JComponent)) {
				continue;
			}
			disableComponentTree((JComponent) child, revert);
		}
	}

	public static String identifierToCaption(String id) {
		StringBuilder result = new StringBuilder();
		int i = 0;
		char lastC = 0;
		for (char c : id.toCharArray()) {
			if (i == 0) {
				result.append(Character.toUpperCase(c));
			} else if (Character.isUpperCase(c) && !Character.isUpperCase(lastC)) {
				result.append(" " + c);
			} else if (Character.isDigit(c) && !Character.isDigit(lastC)) {
				result.append(" " + c);
			} else if (!Character.isLetterOrDigit(c) && Character.isLetterOrDigit(lastC)) {
				result.append(" " + c);
			} else {
				result.append(c);
			}
			lastC = c;
			i++;
		}
		return result.toString();
	}

	public static Rectangle getWindowMaximumBounds() {
		return GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
	}

	public static Rectangle getWindowAverageBounds() {
		Rectangle bounds = getWindowMaximumBounds();
		bounds.x += Math.round(bounds.width / 8f);
		bounds.y += Math.round(bounds.height / 8f);
		bounds.width -= Math.round(bounds.width / 4f);
		bounds.height -= Math.round(bounds.height / 4f);
		return bounds;
	}

	public static boolean runWithTimeout(final Runnable runnable, long timeoutMilliseconds, ExecutorService executor) {
		Runnable runOnInterruptionTimeout = new Runnable() {
			@Override
			public void run() {
				throw getTaskCancellationFailureException("Hang detected");
			}
		};
		return runWithTimeout(runnable, timeoutMilliseconds, 5 * 1000, runOnInterruptionTimeout, executor);
	}

	public static CancellationException getTaskCancellationFailureException(String message) {
		return new CancellationException(message);
	}

	public static TimeoutException getTaskTimeoutException(String message) {
		return new TimeoutException(message);
	}

	public static boolean runWithTimeout(final Runnable runnable, long timeoutMillisecondsToInterrupt,
			final long interruptionTimeoutMillisecondsToDestroy, final Runnable runOnInterruptionTimeout,
			final ExecutorService executor) {
		final boolean[] status = new boolean[1];
		// need to handle task completion reporting here because
		// future.isDone() always returns true after cancel()
		final boolean[] runnableDone = new boolean[] { false };
		Runnable runnableReportingCompletion = new Runnable() {
			@Override
			public void run() {
				runnable.run();
				runnableDone[0] = true;
			}
		};
		Runnable onSuccess = new Runnable() {
			@Override
			public void run() {
				status[0] = true;
			}
		};
		@SuppressWarnings("rawtypes")
		Listener<Future> onTimeout = new Listener<Future>() {
			@Override
			public void handle(Future future) {
				long timedoutSince = System.currentTimeMillis();
				while (!runnableDone[0]) {
					future.cancel(true);
					if ((System.currentTimeMillis() - timedoutSince) > interruptionTimeoutMillisecondsToDestroy) {
						if (runOnInterruptionTimeout != null) {
							runOnInterruptionTimeout.run();
						}
						break;
					}
					relieveCPU();
				}
				status[0] = false;
			}
		};
		Listener<Throwable> onError = new Listener<Throwable>() {
			@Override
			public void handle(Throwable thrown) {
				ErrorManager.rethrow(thrown);
			}
		};
		runWithTimeout(runnableReportingCompletion, timeoutMillisecondsToInterrupt, executor, onSuccess, onTimeout,
				onError);
		return status[0];
	}

	@SuppressWarnings("rawtypes")
	public static void runWithTimeout(final Runnable runnable, long timeoutMilliseconds, ExecutorService executor,
			Runnable onSuccess, Listener<Future> onTimeout, Listener<Throwable> onError) {
		boolean defaultExecutorUsed;
		if (executor == null) {
			defaultExecutorUsed = true;
			final Thread callingThread = Thread.currentThread();
			executor = Executors.newSingleThreadExecutor(new ThreadFactory() {
				@Override
				public Thread newThread(Runnable r) {
					Thread result = new Thread(r, MiscUtils.formatThreadName(MiscUtils.class,
							"DefaultTimeOutMonitor(" + callingThread.getName() + ")"));
					result.setDaemon(true);
					return result;
				}
			});
		} else {
			defaultExecutorUsed = false;
		}
		Future<?> future = executor.submit(runnable);
		try {
			future.get(timeoutMilliseconds, TimeUnit.MILLISECONDS);
			if (onSuccess != null) {
				onSuccess.run();
			}
		} catch (InterruptedException e) {
			if (onError != null) {
				onError.handle(e);
			}
		} catch (ExecutionException e) {
			if (onError != null) {
				onError.handle(e.getCause());
			}
		} catch (TimeoutException e) {
			if (onTimeout != null) {
				onTimeout.handle(future);
			}
		} finally {
			if (defaultExecutorUsed) {
				executor.shutdown();
			}
		}
	}

	public static void repaintImmediately(JComponent c) {
		c.paintImmediately(0, 0, c.getWidth(), c.getHeight());
	}

	public static Window getWindowAncestorOrSelf(Component c) {
		if (c instanceof Window) {
			return (Window) c;
		} else {
			return SwingUtilities.getWindowAncestor(c);
		}
	}

	public static void openMailClient(String targetAddresses, String subject, String mailBody,
			List<File> attachedFiles) {
		if (!Desktop.isDesktopSupported()) {
			throw new UnsupportedOperationException(Desktop.class.getName() + " is not supported");
		}
		Desktop desktop = Desktop.getDesktop();
		if (!desktop.isSupported(Desktop.Action.MAIL)) {
			throw new UnsupportedOperationException(Desktop.class.getName() + " mail action not supported");
		}
		try {
			URI mailto = new URI("mailto:" + targetAddresses + "?subject=" + URLParameterEncoder.encode(subject)
					+ "&body=" + URLParameterEncoder.encode(mailBody));
			desktop.mail(mailto);
		} catch (Exception t) {
			throw new RuntimeException(t);
		}
	}

	public static void runImmediatelyInUIThread(Runnable runnable) {
		if (SwingUtilities.isEventDispatchThread()) {
			runnable.run();
		} else {
			try {
				SwingUtilities.invokeAndWait(runnable);
			} catch (InterruptedException e) {
				throw new UnexpectedError(e);
			} catch (InvocationTargetException e) {
				AbstractApplicationError.rethrow(e.getCause());
			}
		}
	}

	@SuppressWarnings("hiding")
	public static <T> String stringJoin(T[] array, String separator) {
		return stringJoin(Arrays.asList(array), separator);
	}

	public static String stringJoin(List<?> list, String separator) {
		StringBuilder result = new StringBuilder();
		for (int i = 0; i < list.size(); i++) {
			Object item = list.get(i);
			if (i > 0) {
				result.append(separator);
			}
			if (item == null) {
				result.append("null");
			} else {
				result.append(item.toString());
			}
		}
		return result.toString();
	}

	public static <K, V> List<K> getKeysFromValue(Map<K, V> map, Object value) {
		List<K> result = new ArrayList<K>();
		for (Map.Entry<K, V> entry : map.entrySet()) {
			if (equalsOrBothNull(entry.getValue(), value)) {
				result.add(entry.getKey());
			}
		}
		return result;
	}

	public static <K, V> K getFirstKeyFromValue(Map<K, V> map, Object value) {
		List<K> list = getKeysFromValue(map, value);
		if (list.size() > 0) {
			return list.get(0);
		}
		return null;
	}

	public static boolean equalsOrBothNull(Object o1, Object o2) {
		if (o1 == null) {
			return o2 == null;
		} else {
			return o1.equals(o2);
		}
	}

	public static <E> boolean endsWith(List<E> list, List<E> end) {
		if (list.size() < end.size()) {
			return false;
		}
		int listIndex = list.size() - end.size();
		for (E endElement : end) {
			E listElement = list.get(listIndex);
			if (!endElement.equals(listElement)) {
				return false;
			}
			listIndex++;
		}
		return true;
	}

	public static void readAll(InputStream input, OutputStream output) throws IOException {
		int b;
		while (true) {
			b = input.read();
			if (b == -1) {
				break;
			}
			output.write(b);
		}
	}

	@SuppressWarnings("rawtypes")
	public static int readUntil(InputStream input, OutputStream out, final long timeoutMilliseconds,
			Listener<Future> onTimeout, byte[]... endings) throws IOException {
		final Thread callingThread = Thread.currentThread();
		ExecutorService executor = Executors.newSingleThreadExecutor(new ThreadFactory() {
			@Override
			public Thread newThread(Runnable r) {
				Thread result = new Thread(r, MiscUtils.formatThreadName(MiscUtils.class,
						"DefaultTimeOutEnabledStreamReader(" + callingThread.getName() + ")"));
				result.setDaemon(true);
				return result;
			}
		});
		try {
			return readUntil(executor, input, out, timeoutMilliseconds, onTimeout, endings);
		} finally {
			executor.shutdownNow();
		}
	}

	@SuppressWarnings("rawtypes")
	public static int readUntil(ExecutorService executor, InputStream input, OutputStream output,
			final long timeoutMilliseconds, Listener<Future> onTimeout, byte[]... endings) throws IOException {
		final List<List<Byte>> endingList = new ArrayList<List<Byte>>();
		final int[] endingmaxLenth = new int[] { 0 };
		for (byte[] ending : endings) {
			endingList.add(Arrays.asList((Byte[]) PrimitiveUtils.primitiveToWrapperArray(ending)));
			endingmaxLenth[0] = Math.max(endingmaxLenth[0], ending.length);
		}
		final int[] encounterEndingIndex = new int[] { -1 };
		final List<Byte> endingComparaisonBuffer = new ArrayList<Byte>(endingmaxLenth[0]);
		final Throwable[] error = new Throwable[1];
		Runnable runnable = new Runnable() {
			@Override
			public void run() {
				int nextByte;
				while (true) {
					if (Thread.currentThread().isInterrupted()) {
						return;
					}
					try {
						nextByte = input.read();
					} catch (IOException e) {
						error[0] = e;
						return;
					}
					if (nextByte == -1) {
						return;
					}
					try {
						output.write(nextByte);
					} catch (IOException e) {
						throw new UnexpectedError(e);
					}
					if (endingmaxLenth[0] > 0) {
						if (endingComparaisonBuffer.size() == endingmaxLenth[0]) {
							endingComparaisonBuffer.remove((int) 0);
						}
						endingComparaisonBuffer.add((byte) nextByte);
						for (int iEnding = 0; iEnding < endingList.size(); iEnding++) {
							List<Byte> ending = endingList.get(iEnding);
							if (endsWith(endingComparaisonBuffer, ending)) {
								encounterEndingIndex[0] = iEnding;
								return;
							}
						}
					}
				}
			}
		};
		Runnable onSuccess = null;
		Listener<Throwable> onError = new Listener<Throwable>() {
			@Override
			public void handle(Throwable thrown) {
				error[0] = thrown;
			}
		};
		runWithTimeout(runnable, timeoutMilliseconds, executor, onSuccess, onTimeout, onError);
		if (error[0] != null) {
			if (error[0] instanceof IOException) {
				throw (IOException) error[0];
			} else {
				ErrorManager.rethrow(error[0]);
			}
		}
		return encounterEndingIndex[0];
	}

	public static boolean isStreamClosedException(IOException e) {
		return e.toString().toLowerCase().contains("stream closed");
	}

	public static String captureThreadDump() {
		Map<?, ?> allThreads = Thread.getAllStackTraces();
		Iterator<?> iterator = allThreads.keySet().iterator();
		StringBuffer stringBuffer = new StringBuffer();
		while (iterator.hasNext()) {
			Thread key = (Thread) iterator.next();
			StackTraceElement[] trace = (StackTraceElement[]) allThreads.get(key);
			stringBuffer.append(key + "\n");
			for (int i = 0; i < trace.length; i++) {
				stringBuffer.append(" " + trace[i] + "\n");
			}
			stringBuffer.append("\n");
		}
		return stringBuffer.toString();
	}

	public static boolean isEventDispatchThreadBusy() {
		return Toolkit.getDefaultToolkit().getSystemEventQueue().peekEvent() != null;
	}

	public static boolean hasOrContainsFocus(Component c) {
		if (c.isFocusOwner()) {
			return true;
		}
		if (c instanceof Container) {
			Container container = (Container) c;
			for (Component childComponent : container.getComponents()) {
				if (hasOrContainsFocus(childComponent)) {
					return true;
				}
			}
		}
		return false;
	}

	public static void forceUpdateConstant(Field field, Object newValue) throws Exception {
		field.setAccessible(true);
		Field modifiersField = Field.class.getDeclaredField("modifiers");
		modifiersField.setAccessible(true);
		modifiersField.setInt(field, field.getModifiers() & ~Modifier.FINAL);
		field.set(null, newValue);
	}

	public static int getStandardCharacterWidth(Component c) {
		Font font = c.getFont();
		if (font == null) {
			font = UIManager.getFont("Panel.font");
		}
		return c.getFontMetrics(font).charWidth('a');
	}

	public static void openWebPage(String url) {
		URI uri;
		try {
			uri = new URL(url).toURI();
		} catch (Exception e) {
			throw new UnexpectedError("URL error: '" + url + "': " + e, e);
		}
		try {
			Desktop.getDesktop().browse(uri);
		} catch (Exception e) {
			if (!new AlternativeDesktopApi() {

				@Override
				protected void logErr(String msg, Throwable t) {
				}

				@Override
				protected void logErr(String msg) {
				}

				@Override
				protected void logOut(String msg) {
				}

			}.browse(uri)) {
				throw new UnexpectedError("Failed to display the web page '" + url + "': " + e, e);
			}
		}
	}

	public static void setSplitPaneDividerLocationBeforeRealization(JSplitPane splitPane, double dividerLocationRatio) {
		splitPane.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent ce) {
				splitPane.removeComponentListener(this);
				splitPane.setDividerLocation(dividerLocationRatio);
			}
		});
		splitPane.setResizeWeight(dividerLocationRatio);
	}

	public static void clearCurrentThreadInterruptedStatus() {
		Thread.interrupted();
	}

	public static void overrideNimbusClientProperty(JComponent c, String propertyName, Object propertyValue) {
		String NIMBUS_OVERRIDE_KEY = "Nimbus.Overrides";
		UIDefaults uiDefaults = (UIDefaults) c.getClientProperty(NIMBUS_OVERRIDE_KEY);
		if (uiDefaults == null) {
			uiDefaults = new UIDefaults();
		}
		if (propertyValue == null) {
			uiDefaults.remove(propertyName);
		} else {
			uiDefaults.put(propertyName, propertyValue);
		}
		c.putClientProperty(NIMBUS_OVERRIDE_KEY, uiDefaults);
		SwingUtilities.updateComponentTreeUI(c);
	}

	public static void setCrossLookAndFeelBackground(JPanel panel, Color bg) {
		panel.setBackground(bg);
		if (UIManager.getLookAndFeel() instanceof NimbusLookAndFeel) {
			overrideNimbusClientProperty(panel, "Panel.background", bg);
		}
	}

	public static void setCrossLookAndFeelBackground(JTextArea textArea, final Color bg) {
		textArea.setBackground(bg);
		if (UIManager.getLookAndFeel() instanceof NimbusLookAndFeel) {
			for (String state : Arrays.asList("Enabled", "Disabled", "Enabled+NotInScrollPane",
					"Disabled+NotInScrollPane")) {
				overrideNimbusClientProperty(textArea, "TextArea[" + state + "].backgroundPainter",
						(bg == null) ? null : new javax.swing.plaf.nimbus.AbstractRegionPainter() {
							@Override
							protected javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext getPaintContext() {
								return new javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext(null, null,
										false);
							}

							@Override
							protected void doPaint(Graphics2D g, JComponent c, int width, int height,
									Object[] extendedCacheKeys) {
								g.setColor(bg);
								g.fill(new Rectangle(0, 0, width, height));
							}
						});
			}
		}
	}

	public static void setCrossLookAndFeelBackground(JButton button, final Color bg) {
		button.setBackground(bg);
		if (UIManager.getLookAndFeel() instanceof NimbusLookAndFeel) {
			overrideNimbusClientProperty(button, "Button[Enabled].backgroundPainter",
					(bg == null) ? null : new javax.swing.plaf.nimbus.AbstractRegionPainter() {
						@Override
						protected javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext getPaintContext() {
							return new javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext(null, null, false);
						}

						@Override
						protected void doPaint(Graphics2D g, JComponent c, int width, int height,
								Object[] extendedCacheKeys) {
							g.setColor(bg);
							g.fill(new Rectangle(0, 0, width, height));
						}
					});
		}
	}

	public static void setCrossLookAndFeelBackground(JToggleButton toogleButton, final Color bg) {
		toogleButton.setBackground(bg);
		if (UIManager.getLookAndFeel() instanceof NimbusLookAndFeel) {
			overrideNimbusClientProperty(toogleButton, "ToggleButton[Enabled].backgroundPainter",
					(bg == null) ? null : new javax.swing.plaf.nimbus.AbstractRegionPainter() {
						@Override
						protected javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext getPaintContext() {
							return new javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext(null, null, false);
						}

						@Override
						protected void doPaint(Graphics2D g, JComponent c, int width, int height,
								Object[] extendedCacheKeys) {
							g.setColor(bg);
							g.fill(new Rectangle(0, 0, width, height));
						}
					});
		}
	}

	public static void setCrossLookAndFeelBackground(JTextField textField, Color bg) {
		textField.setBackground(bg);
		if (UIManager.getLookAndFeel() instanceof NimbusLookAndFeel) {
			overrideNimbusClientProperty(textField, "TextField[Enabled].backgroundPainter",
					(bg == null) ? null : new javax.swing.plaf.nimbus.AbstractRegionPainter() {
						@Override
						protected javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext getPaintContext() {
							return new javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext(null, null, false);
						}

						@Override
						protected void doPaint(Graphics2D g, JComponent c, int width, int height,
								Object[] extendedCacheKeys) {
							g.setColor(bg);
							g.fill(new Rectangle(0, 0, width, height));
						}
					});
		}
	}

	public static void removeCrossLookAndFeelBorder(JScrollPane scrollPane) {
		scrollPane.setBorder(null);
		if (UIManager.getLookAndFeel() instanceof NimbusLookAndFeel) {
			overrideNimbusClientProperty(scrollPane, "ScrollPane[Enabled].borderPainter",
					new javax.swing.plaf.nimbus.AbstractRegionPainter() {
						@Override
						protected javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext getPaintContext() {
							return new javax.swing.plaf.nimbus.AbstractRegionPainter.PaintContext(null, null, false);
						}

						@Override
						protected void doPaint(Graphics2D g, JComponent c, int width, int height,
								Object[] extendedCacheKeys) {
						}
					});
		}
	}

	public static String toHTML(Color color) {
		return String.format("#%02x%02x%02x", color.getRed(), color.getGreen(), color.getBlue());
	}

	@SuppressWarnings("unchecked")
	public static <I> I implementCompatibleInterface(Object object, Class<I> theInterface) {
		return (I) Proxy.newProxyInstance(object.getClass().getClassLoader(), new Class<?>[] { theInterface },
				new InvocationHandler() {

					@Override
					public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
						return object.getClass().getMethod(method.getName(), method.getParameterTypes()).invoke(object,
								args);
					}
				});
	}

	public static OutputStream getLineListenerStream(Listener<String> lineListener) {
		return new OutputStream() {

			ByteArrayOutputStream lineBuffer = new ByteArrayOutputStream();
			int lastByte = -1;

			@Override
			public void write(int currentByte) throws IOException {
				try {
					if (currentByte == -1) {
						flush();
						return;
					}
					if (currentByte == (int) '\n') {
						if (lastByte == (int) '\r') {
							return;
						}
						flush();
						return;
					}
					if (currentByte == (int) '\r') {
						flush();
						return;
					}
					lineBuffer.write(currentByte);
				} finally {
					lastByte = currentByte;
				}
			}

			@Override
			public void flush() throws IOException {
				lineListener.handle(lineBuffer.toString());
				lineBuffer.reset();
			}

		};
	}

	public static <C> C takeUntilLast(BlockingQueue<C> queue) {
		C result = null;
		while (queue.size() > 0) {
			try {
				result = queue.take();
			} catch (InterruptedException e) {
				throw new UnexpectedError(e);
			}
		}
		return result;
	}

	public static <C> C removeUntilLast(Queue<C> queue) {
		C result = null;
		while (queue.size() > 0) {
			result = queue.remove();
		}
		return result;
	}

	public static BufferedImage extractOutOfBoundsImage(BufferedImage image, Rectangle bounds) {
		BufferedImage result = new BufferedImage(bounds.width, bounds.height, ImageUtils.getAdaptedBufferedImageType());
		Graphics2D g2d = result.createGraphics();
		int dx1 = 0;
		int dy1 = 0;
		int dx2 = bounds.width;
		int dy2 = bounds.height;
		int sx1 = bounds.x;
		int sy1 = bounds.y;
		int sx2 = bounds.x + bounds.width;
		int sy2 = bounds.y + bounds.height;
		g2d.drawImage(image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null, null);
		g2d.dispose();
		return result;
	}

	public static BufferedImage rotatePreservingDimensions(BufferedImage image, int angleDegrees, Point rotationCenter,
			boolean smoothInterpolation) {
		int w = image.getWidth();
		int h = image.getHeight();
		BufferedImage result = new BufferedImage(w, h, ImageUtils.getAdaptedBufferedImageType());
		Graphics2D g = result.createGraphics();
		if (smoothInterpolation) {
			ImageUtils.setSmoothScalingRenderingHints(g);
		}
		double radians = -Math.toRadians(angleDegrees);
		AffineTransform transform = new AffineTransform();
		transform.rotate(radians, rotationCenter.x, rotationCenter.y);
		g.setTransform(transform);
		g.drawImage(image, 0, 0, null);
		g.dispose();
		return result;
	}

	public static String hexEncode(String s) {
		return hexEncode(s.getBytes());
	}

	public static String hexEncode(byte[] byteArray) {
		StringBuilder result = new StringBuilder();
		for (byte b : byteArray) {
			result.append(Integer.toHexString(MathUtils.unsignedByte(b)));
		}
		return result.toString();
	}

	public static InputStream sendHttpPost(URL url, Map<String, String> headers, String postBody) throws IOException {
		URLConnection con = url.openConnection();
		HttpURLConnection http = (HttpURLConnection) con;
		http.setRequestMethod("POST");
		http.setDoOutput(true);
		byte[] post = postBody.getBytes(StandardCharsets.UTF_8);
		int length = post.length;
		http.setFixedLengthStreamingMode(length);
		for (Map.Entry<String, String> entry : headers.entrySet()) {
			http.setRequestProperty(entry.getKey(), entry.getValue());
		}
		http.connect();
		OutputStream outputStream = http.getOutputStream();
		outputStream.write(post);
		return http.getInputStream();
	}

	public static InputStream sendHttpPost(URL url, Map<String, String> headers, Map<String, String> arguments)
			throws IOException {
		StringJoiner postBuilder = new StringJoiner("&");
		for (Map.Entry<String, String> entry : arguments.entrySet()) {
			postBuilder.add(
					URLEncoder.encode(entry.getKey(), "UTF-8") + "=" + URLEncoder.encode(entry.getValue(), "UTF-8"));
		}
		String postBody = postBuilder.toString();
		return sendHttpPost(url, headers, postBody);
	}

	public static InputStream sendHttpPost(URL url, Map<String, String> arguments) throws IOException {
		return sendHttpPost(url,
				Collections.singletonMap("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8"),
				arguments);
	}

	public static String encrypt(String s) {
		try {
			String algorithm = "RC4";
			String secret = "secret";
			SecretKeySpec keySpec = new SecretKeySpec(secret.getBytes(), algorithm);
			Cipher cipher = Cipher.getInstance(algorithm);
			cipher.init(Cipher.ENCRYPT_MODE, keySpec);
			return hexEncode(cipher.doFinal(s.getBytes()));
		} catch (Exception e) {
			throw new UnexpectedError("License key encryption error", e);
		}
	}

	public static ExecutorService newExecutor(final String threadNamePrefix, int minimumThreadCount,
			final boolean daemonThreads) {
		ThreadPoolExecutor result = new ThreadPoolExecutor(minimumThreadCount, Integer.MAX_VALUE, 60000,
				TimeUnit.MILLISECONDS, new SynchronousQueue<Runnable>(), new ThreadFactory() {
					int threadNumber = 0;

					@Override
					public Thread newThread(Runnable r) {
						Thread result = new Thread(r);
						result.setName(threadNamePrefix + "-" + Integer.toString(threadNumber++));
						result.setDaemon(daemonThreads);
						return result;
					}
				});
		return result;
	}

}
