package com.otk.application.util.draw;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

import com.otk.application.error.UnexpectedError;
import com.otk.application.util.ImageUtils;

public class DropShadow {

	public static final BufferedImage FRAME;
	static {
		try {
			FRAME = ImageUtils.loadAdaptedImage(DropShadow.class.getResourceAsStream("dropShadow.png"));
		} catch (Throwable t) {
			throw new UnexpectedError(t);
		}
	}

	public static BufferedImage add(BufferedImage image) {
		BufferedImage result = new BufferedImage(image.getWidth(), image.getHeight(),
				ImageUtils.getAdaptedBufferedImageType());
		Graphics2D g = result.createGraphics();
		ImageUtils.setSmoothScalingRenderingHints(g);
		drawWith(g, image, new Rectangle(0, 0, result.getWidth(), result.getHeight()));
		g.dispose();
		return result;
	}

	public static void drawWith(Graphics2D g, BufferedImage image, Rectangle bounds) {
		g.drawImage(FRAME, bounds.x, bounds.y, bounds.width, bounds.height, null);
		Rectangle innerArea = getInnerArea(bounds);
		g.drawImage(image, innerArea.x, innerArea.y, innerArea.width, innerArea.height, null);
	}

	public static Rectangle getInnerArea(Rectangle bounds) {
		int x = bounds.x + Math.round(bounds.width * 0.02f);
		int y = bounds.y + Math.round(bounds.height * 0.02f);
		int width = Math.round(bounds.width * 0.95f);
		int height = Math.round(bounds.height * 0.95f);
		return new Rectangle(x, y, width, height);
	}

}
