package com.otk.application.util.draw;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

import com.otk.application.image.camera.FrameFormat;
import com.otk.application.image.filter.AbstractFilter;
import com.otk.application.image.filter.AbstractHeadlessFilter;
import com.otk.application.image.filter.BoundingBox;
import com.otk.application.image.filter.FilteringContext;
import com.otk.application.util.ImageUtils;
import com.otk.application.util.MathUtils;

public class FaceFrame {

	private static final Image HEAD_SHAPE = ImageUtils
			.requireAdaptedImage(FaceFrame.class.getResourceAsStream("faceFramePositioningHeadShape.png"));
	private static final Image RULERS = ImageUtils
			.requireAdaptedImage(FaceFrame.class.getResourceAsStream("faceFramePositioningRulers.png"));
	private static final Image YELLOW_SQUARE = ImageUtils
			.requireAdaptedImage(FaceFrame.class.getResourceAsStream("faceFramePositioningYellowSquare.png"));
	public final AbstractFilter filter = new AbstractHeadlessFilter() {

		@Override
		public FilteringContext doExecute(FilteringContext context) {
			BufferedImage newImage = ImageUtils.copy(context.getImage());
			Graphics2D g2d = newImage.createGraphics();
			paint(g2d, newImage.getWidth(), newImage.getHeight());
			g2d.dispose();
			return context.withImage(newImage);
		}

	};

	public FaceFrame() {
	}

	private double widthOverHeightRatio = 35.0 / 45.0;
	private double heightRatio = 0.6;
	private double horizontalOffsetRatio;
	private double verticalOffsetRatio = 0.0;
	private double borderWidthRatio = 0.01;
	private boolean positioningOverlayHeadShapeVisible = true;
	private boolean positioningOverlayRulersVisible = true;
	private boolean positioningOverlayYellowSquareVisible;
	private double positioningOverlayVerticalOffsetRatio = 0.0;
	private double positioningOverlayHeightRatio = 0.7;
	private double positioningOverlayWidthRatio = 0.9;

	public double getBorderWidthRatio() {
		return borderWidthRatio;
	}

	public void setBorderWidthRatio(double borderWidthRatio) {
		this.borderWidthRatio = borderWidthRatio;
	}

	public double getPositioningOverlayVerticalOffsetRatio() {
		return positioningOverlayVerticalOffsetRatio;
	}

	public void setPositioningOverlayVerticalOffsetRatio(double positioningOverlayVerticalOffsetRatio) {
		this.positioningOverlayVerticalOffsetRatio = positioningOverlayVerticalOffsetRatio;
	}

	public double getPositioningOverlayHeightRatio() {
		return positioningOverlayHeightRatio;
	}

	public void setPositioningOverlayHeightRatio(double positioningOverlayHeightRatio) {
		this.positioningOverlayHeightRatio = positioningOverlayHeightRatio;
	}

	public double getPositioningOverlayWidthRatio() {
		return positioningOverlayWidthRatio;
	}

	public void setPositioningOverlayWidthRatio(double positioningOverlayWidthRatio) {
		this.positioningOverlayWidthRatio = positioningOverlayWidthRatio;
	}

	public double getHeightRatio() {
		return heightRatio;
	}

	public void setHeightRatio(double heightRatio) {
		this.heightRatio = heightRatio;
	}

	public double getHorizontalOffsetRatio() {
		return horizontalOffsetRatio;
	}

	public void setHorizontalOffsetRatio(double horizontalOffsetRatio) {
		this.horizontalOffsetRatio = horizontalOffsetRatio;
	}

	public double getVerticalOffsetRatio() {
		return verticalOffsetRatio;
	}

	public void setVerticalOffsetRatio(double verticalOffsetRatio) {
		this.verticalOffsetRatio = verticalOffsetRatio;
	}

	public boolean isPositioningOverlayYellowSquareVisible() {
		return positioningOverlayYellowSquareVisible;
	}

	public void setPositioningOverlayYellowSquareVisible(boolean positioningOverlayYellowSquareVisible) {
		this.positioningOverlayYellowSquareVisible = positioningOverlayYellowSquareVisible;
	}

	public boolean isPositioningOverlayHeadShapeVisible() {
		return positioningOverlayHeadShapeVisible;
	}

	public void setPositioningOverlayHeadShapeVisible(boolean positioningOverlayHeadShapeVisible) {
		this.positioningOverlayHeadShapeVisible = positioningOverlayHeadShapeVisible;
	}

	public boolean isPositioningOverlayRulersVisible() {
		return positioningOverlayRulersVisible;
	}

	public void setPositioningOverlayRulersVisible(boolean positioningOverlayRulersVisible) {
		this.positioningOverlayRulersVisible = positioningOverlayRulersVisible;
	}

	public double getWidthOverHeightRatio() {
		return widthOverHeightRatio;
	}

	public void setWidthOverHeightRatio(double widthOverHeightRatio) {
		this.widthOverHeightRatio = widthOverHeightRatio;
	}

	public AbstractFilter getFilter() {
		return filter;
	}

	private void paintHeadShape(Graphics g, int canvasWidth, int canvasHeight) {
		Rectangle bounds = getPositioningOverlayBounds(canvasWidth, canvasHeight);
		paintHeadShape(g, bounds);
	}

	private void paintHeadShape(Graphics g, Rectangle bounds) {
		paintPositioningOverlay(HEAD_SHAPE, g, bounds);
	}

	private void paintRulers(Graphics g, int canvasWidth, int canvasHeight) {
		Rectangle bounds = getPositioningOverlayBounds(canvasWidth, canvasHeight);
		paintRulers(g, bounds);
	}

	private void paintBorder(Graphics2D g, int canvasWidth, int canvasHeight) {
		Rectangle bounds = getBounds(canvasWidth, canvasHeight);
		paintBorder(g, bounds);
	}

	private void paintRulers(Graphics g, Rectangle bounds) {
		paintPositioningOverlay(RULERS, g, bounds);
	}

	private void paintBorder(Graphics2D g, Rectangle bounds) {
		int borderLineWidth = MathUtils.round(getBorderWidthRatio() * (bounds.width + bounds.height) / 2.0);
		int x = bounds.x - borderLineWidth;
		int y = bounds.y - borderLineWidth;
		int width = bounds.width + 2 * borderLineWidth;
		int height = bounds.height + 2 * borderLineWidth;
		g.setStroke(new BasicStroke(borderLineWidth));
		g.setColor(Color.RED);
		g.drawRect(x, y, width, height);
	}

	private void paintPositioningOverlay(Image overlayImage, Graphics g, Rectangle bounds) {
		Graphics2D g2d = (Graphics2D) g;
		AlphaComposite newComposite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f);
		Composite compositeTorestore = g2d.getComposite();
		g2d.setComposite(newComposite);
		ImageUtils.drawSmoothlyScaled(g2d, overlayImage, bounds.x, bounds.y, bounds.width, bounds.height);
		g2d.setComposite(compositeTorestore);
	}

	public BufferedImage crop(Image picture) {
		BufferedImage bufferedPicture = ImageUtils.getBufferedImage(picture);
		Rectangle croppingRect = getBounds(bufferedPicture.getWidth(), bufferedPicture.getHeight());
		BufferedImage result = new BufferedImage(croppingRect.width, croppingRect.height,
				ImageUtils.getAdaptedBufferedImageType());
		Graphics2D g2d = result.createGraphics();
		int dx1 = 0;
		int dx2 = croppingRect.width;
		int dy1 = 0;
		int dy2 = croppingRect.height;
		int sx1 = croppingRect.x;
		int sx2 = croppingRect.x + croppingRect.width;
		int sy1 = croppingRect.y;
		int sy2 = croppingRect.y + croppingRect.height;
		g2d.drawImage(bufferedPicture, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null);
		g2d.dispose();
		return result;
	}

	public Rectangle getBounds(int canvasWidth, int canvasHeight) {
		double widthRatio = getStandardizingWidthRatio(canvasWidth, canvasHeight);
		return getBounds(canvasWidth, canvasHeight, widthRatio, getHeightRatio(), getHorizontalOffsetRatio(),
				getVerticalOffsetRatio());
	}

	public boolean isOutOfbounds(int canvasWidth, int canvasHeight) {
		Rectangle bounds = getBounds(canvasWidth, canvasHeight);
		return !new Rectangle(canvasWidth, canvasHeight).contains(bounds);
	}

	public double getStandardizingWidthRatio(double canvasWidth, double canvasHeight) {
		double height = canvasHeight * getHeightRatio();
		double width = height * getWidthOverHeightRatio();
		return width / canvasWidth;
	}

	public double getHeightRatio(int canvasWidth, int canvasHeight, double widthRatio) {
		double width = canvasWidth * widthRatio;
		double height = width / getWidthOverHeightRatio();
		return height / canvasHeight;
	}

	private Rectangle getBounds(int canvasWidth, int canvasHeight, double widthRatio, double heightRatio,
			double horizontalOffsetRatio, double verticalOffsetRatio) {
		double centerX = (0.5 + horizontalOffsetRatio) * canvasWidth;
		double centerY = (0.5 + verticalOffsetRatio) * canvasHeight;
		double width = canvasWidth * widthRatio;
		double height = canvasHeight * heightRatio;
		Rectangle result = MathUtils.getBoundsAroundCenter(centerX, centerY, width, height).getBounds();
		return result;
	}

	private Rectangle getFrameRelativePositioningOverlayBounds(int frameWidth, int frameHeight) {
		return getBounds(frameWidth, frameHeight, getPositioningOverlayWidthRatio(), getPositioningOverlayHeightRatio(),
				0.0, getPositioningOverlayVerticalOffsetRatio());
	}

	public Rectangle getPositioningOverlayBounds(int canvasWidth, int canvasHeight) {
		Rectangle frameBounds = getBounds(canvasWidth, canvasHeight);
		Rectangle result = getFrameRelativePositioningOverlayBounds(frameBounds.width, frameBounds.height);
		result.x += frameBounds.x;
		result.y += frameBounds.y;
		return result;
	}

	private Dimension getCroppingSize(int canvasWidth, int canvasHeight) {
		return getBounds(canvasWidth, canvasHeight).getSize();
	}

	public void paint(Graphics2D g2d, int canvasWidth, int canvasHeight) {
		if (isPositioningOverlayHeadShapeVisible()) {
			paintHeadShape(g2d, canvasWidth, canvasHeight);
		}
		if (isPositioningOverlayRulersVisible()) {
			paintRulers(g2d, canvasWidth, canvasHeight);
		}
		if (isPositioningOverlayYellowSquareVisible()) {
			paintYellowSquare(g2d, canvasWidth, canvasHeight);
		}
		paintBorder(g2d, canvasWidth, canvasHeight);
	}

	public void paintPositioningOverlayOnCroppedImage(Graphics2D g2d, Dimension croppingSize) {
		Rectangle overlayBounds = getFrameRelativePositioningOverlayBounds(croppingSize.width, croppingSize.height);
		if (isPositioningOverlayHeadShapeVisible()) {
			paintHeadShape(g2d, overlayBounds);
		}
		if (isPositioningOverlayRulersVisible()) {
			paintRulers(g2d, overlayBounds);
		}
		if (isPositioningOverlayYellowSquareVisible()) {
			paintYellowSquare(g2d, overlayBounds);
		}
	}

	private void paintYellowSquare(Graphics2D g2d, int canvasWidth, int canvasHeight) {
		Rectangle bounds = getPositioningOverlayBounds(canvasWidth, canvasHeight);
		paintYellowSquare(g2d, bounds);
	}

	private void paintYellowSquare(Graphics2D g2d, Rectangle bounds) {
		paintPositioningOverlay(YELLOW_SQUARE, g2d, bounds);

	}

	public BufferedImage correctSize(Image image, Dimension destSize, Color backgroundColor) {
		BoundingBox filter = new BoundingBox();
		filter.width = destSize.width;
		filter.height = destSize.height;
		filter.cutEmptySides = false;
		filter.scaleToHeight = false;
		filter.scaleToWidth = true;
		BufferedImage result = ImageUtils
				.filter(new FilteringContext().withImage(ImageUtils.getBufferedImage(image)), filter).getImage();
		if (backgroundColor != null) {
			result = ImageUtils.changeColor(result, new Color(0, 0, 0, 0), backgroundColor);
		}
		return result;
	}

	public BufferedImage correctCroppedImageSize(Image croppedImage, int canvasWidth, int canvasHeight,
			Color backgroundColor) {
		Dimension faceFrameSize = getCroppingSize(canvasWidth, canvasHeight);
		return correctSize(croppedImage, faceFrameSize, backgroundColor);
	}

	public Dimension getSize(FrameFormat frameFormat) {
		return getCroppingSize(frameFormat.width, frameFormat.height);
	}

}
