package com.otk.application.window;

import java.awt.HeadlessException;

import javax.swing.JFrame;

import com.otk.application.AbstractApplication;
import com.otk.application.util.MiscUtils;

public class ApplicationFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	private boolean disposed = false;
	private AbstractApplication application;

	public ApplicationFrame(AbstractApplication application) throws HeadlessException {
		super();
		this.application = application;
		onInitialization();
		setTitle(getApplication().getName());
		setIconImage(getApplication().getIconImage());
		setBounds(MiscUtils.getWindowAverageBounds());
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}

	public boolean isDisposed() {
		return disposed;
	}

	@Override
	public void dispose() {
		if (!disposed) {
			onDispose();
			super.dispose();
			disposed = true;
		}
	}

	protected void onInitialization() {
		getApplication().getLog().info("Initializing " + this);
		application.getAnalytics().track("Initializing " + this);
	}

	protected void onDispose() {
		getApplication().getLog().info("Destroying " + this);
		application.getAnalytics().track("Destroying " + this);
	}

	public AbstractApplication getApplication() {
		return application;
	}

	@Override
	public String toString() {
		return (getClass().isAnonymousClass() ? getClass().getName() : getClass().getSimpleName()) + " (id="
				+ hashCode() + ")";
	}

}
