package com.otk.application.window;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.MessageFormat;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import com.otk.application.AbstractApplication;
import com.otk.application.translation.T;
import com.otk.application.util.FileUtils;
import com.otk.application.util.MiscUtils;
import com.otk.application.util.TextUtils;

public class UnexpectedErrorDialog extends ApplicationDialog {

	private static final long serialVersionUID = 1L;

	private final JPanel contentPanel;
	private Throwable error;
	private JButton okButton;

	public static void main(String[] args) {
		UnexpectedErrorDialog dialog = new UnexpectedErrorDialog(AbstractApplication.createTestApplication(), null,
				new Exception("bla bla bla\nbla bla bla"));
		dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		dialog.setVisible(true);
	}

	public static void show(AbstractApplication application, Component parent, Throwable error) {
		UnexpectedErrorDialog dialog = new UnexpectedErrorDialog(application, parent, error);
		dialog.setVisible(true);
	}

	public UnexpectedErrorDialog(AbstractApplication application, Component parent, Throwable error) {
		super(application, (parent != null) ? MiscUtils.getWindowAncestorOrSelf(parent) : null);
		setIconImage(application.getIconImage());
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		this.error = error;
		setModal(true);
		setBounds(MiscUtils.getWindowMaximumBounds());
		BorderLayout borderLayout = new BorderLayout();
		getContentPane().setLayout(borderLayout);
		contentPanel = new JPanel();
		{
			contentPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
			getContentPane().add(contentPanel, BorderLayout.CENTER);
			contentPanel.setLayout(new BorderLayout(10, 10));
			JLabel subTitleControl = new JLabel(getSubTitle());
			{
				subTitleControl.setIcon(UIManager.getIcon("OptionPane.errorIcon"));
				subTitleControl.setHorizontalAlignment(SwingConstants.CENTER);
				contentPanel.add(subTitleControl, BorderLayout.NORTH);
			}
			JPanel messagesPanel = new JPanel();
			{
				contentPanel.add(messagesPanel, BorderLayout.CENTER);
				messagesPanel.setLayout(new BoxLayout(messagesPanel, BoxLayout.Y_AXIS));
				{
					JTextArea messageControl = new JTextArea(getApplication().getErrorManager().formatError(error));
					{
						messageControl.setWrapStyleWord(true);
						messageControl.setLineWrap(true);
						messageControl.setOpaque(false);
						messageControl.setEditable(false);
						JScrollPane scrollPane = new JScrollPane(messageControl);
						scrollPane.setBorder(BorderFactory.createTitledBorder(getMessageAreaCaption()));
						Dimension minimumSize = messageControl.getPreferredSize();
						{
							minimumSize.height += 50;
							minimumSize.height = Math.min(minimumSize.height,
									MiscUtils.getWindowAverageBounds().height / 2);
							scrollPane.setMinimumSize(minimumSize);
						}
						messagesPanel.add(scrollPane);
					}
					JTextArea detailsControl = new JTextArea(previewDetailedErrorReport());
					{
						detailsControl.setLineWrap(true);
						detailsControl.setWrapStyleWord(true);
						detailsControl.setEditable(false);
						JScrollPane scrollPane = new JScrollPane(detailsControl);
						scrollPane.setBorder(BorderFactory.createTitledBorder(getDetailsAreaCaption()));
						messagesPanel.add(scrollPane);
					}
					JTextArea adviseControl = new JTextArea(getAdvise());
					{
						messagesPanel.add(adviseControl);
						adviseControl.setWrapStyleWord(true);
						adviseControl.setLineWrap(true);
						adviseControl.setEditable(false);
						MiscUtils.setCrossLookAndFeelBackground(adviseControl, Color.RED);
						adviseControl.setForeground(Color.WHITE);
						TextUtils.changeFontSize(adviseControl, adviseControl.getFont().getSize() + 2);
					}
				}

			}
		}
		JPanel buttonPane = new JPanel();
		{
			buttonPane.setLayout(new FlowLayout(FlowLayout.CENTER));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			okButton = new JButton(getOKButtonCaption());
			{
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						onOK();
					}
				});
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
		}
	}

	@Override
	protected void onInitialization() {
		try {
			super.onInitialization();
		} catch (Throwable ignore) {
		}
	}

	@Override
	protected void onDispose() {
		try {
			super.onDispose();
		} catch (Throwable ignore) {
		}
	}

	protected String getReportSendingEnablementCaption() {
		return T.t("Send this report. Enter your email address (to get feedback) :");
	}

	protected String getAdvise() {
		return MessageFormat.format(T.t("Make sure that you have installed the latest version of {0}.") + "\n"
				+ T.t("Contact the support if the problem persists."), getApplication().getName());
	}

	protected String getDetailsAreaCaption() {
		return T.t("Details");
	}

	protected String getMessageAreaCaption() {
		return T.t("Message");
	}

	protected String getSubTitle() {
		return T.t("An Error Occurred");
	}

	protected String getReportSendingMessage() {
		return T.t("Sending The Report...");
	}

	protected String getOKButtonCaption() {
		return T.t("OK");
	}

	private String previewDetailedErrorReport() {
		StringBuilder result = new StringBuilder();
		result.append("- Configuration:\n"
				+ TextUtils.truncateNicely(TextUtils.toText(getApplication().getConfiguration().asProperties()), 1000));
		result.append("\n\n");
		result.append("- Stack Trace:\n" + TextUtils.truncateNicely(MiscUtils.getPrintedStackTrace(error), 1000));
		result.append("\n\n");
		result.append("- Environment:\n" + TextUtils.truncateNicely(TextUtils.toText(System.getProperties()), 1000));
		File lastLogsFile = new File(getApplication().getConfiguration().getFilePath());
		if (lastLogsFile.exists()) {
			try {
				result.append("\n\n");
				result.append("- Last logs:\n" + TextUtils.truncateNicely(FileUtils.read(lastLogsFile), 1000));
			} catch (Exception e) {
				getApplication().getLog().error(e);
			}
		}
		return result.toString();
	}

	protected void onOK() {
		dispose();
	}

}
