package com.otk.application.window.component;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.text.NumberFormat;
import java.util.Dictionary;
import java.util.Hashtable;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.otk.application.error.UnexpectedError;
import com.otk.application.util.MiscUtils;

public class JSlinner extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private static final int SLIDER_MIN = 0;
	private static final int SLIDER_MAX = 100;

	private JSlider slider;
	private JSpinner spinner;
	private boolean evnetsDisabled = false;
	private float majorToMajorTickSpacingRatio = 0.1f;
	private float minorToMajorTickSpacingRatio = 0.01f;

	public JSlinner() {
		this(new SpinnerNumberModel(50, 0, 100, 1));
	}

	public JSlinner(SpinnerNumberModel model) {
		setLayout(new BorderLayout());
		slider = createSlider();
		spinner = createSpinner();
		setModel(model);
	}

	protected JSpinner createSpinner() {
		JSpinner spinner = new JSpinner();
		add(MiscUtils.flowInLayout(spinner, GridBagConstraints.CENTER), BorderLayout.WEST);
		spinner.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if (evnetsDisabled) {
					return;
				}
				evnetsDisabled = true;
				try {
					slider.setValue(convertSpinnerToSliderValue((Number) spinner.getValue()));
				} finally {
					evnetsDisabled = false;
				}
			}
		});
		return spinner;
	}

	protected JSlider createSlider() {
		JSlider slider = new JSlider();
		add(slider, BorderLayout.CENTER);
		slider.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				if (evnetsDisabled) {
					return;
				}
				evnetsDisabled = true;
				try {
					spinner.setValue(convertSliderToSpinnerValue(slider.getValue()));
				} finally {
					evnetsDisabled = false;
				}
			}
		});
		return slider;
	}

	protected Object convertSliderToSpinnerValue(int value) {
		Number spinnerMin = (Number) getModel().getMinimum();
		Number spinnerMax = (Number) getModel().getMaximum();
		double spinnerExtent = spinnerMax.doubleValue() - spinnerMin.doubleValue();
		int sliderExtent = SLIDER_MAX - SLIDER_MIN;
		double floatingResult = spinnerMin.doubleValue() + ((double) value / (double) sliderExtent) * spinnerExtent;
		if (getModel().getNumber() instanceof Integer) {
			return Math.round((float) floatingResult);
		} else if (getModel().getNumber() instanceof Long) {
			return Math.round(floatingResult);
		} else if (getModel().getNumber() instanceof Float) {
			return (float) floatingResult;
		} else if (getModel().getNumber() instanceof Double) {
			return floatingResult;
		} else {
			throw new UnexpectedError(SpinnerNumberModel.class.getSimpleName() + " type not handled: "
					+ getModel().getNumber().getClass());
		}
	}

	protected int convertSpinnerToSliderValue(Number value) {
		Number spinnerMin = (Number) getModel().getMinimum();
		Number spinnerMax = (Number) getModel().getMaximum();
		float spinnerExtent = spinnerMax.floatValue() - spinnerMin.floatValue();
		int sliderExtent = SLIDER_MAX - SLIDER_MIN;
		float floatingResult = sliderExtent * (value.floatValue() - spinnerMin.floatValue()) / spinnerExtent;
		return Math.round(floatingResult);
	}

	public SpinnerNumberModel getModel() {
		return (SpinnerNumberModel) spinner.getModel();
	}

	public void setModel(SpinnerNumberModel model) {
		spinner.setModel(model);
		slider.setMinimum(SLIDER_MIN);
		slider.setMaximum(SLIDER_MAX);
		slider.setValue(convertSpinnerToSliderValue((Number) model.getValue()));
		slider.setMinorTickSpacing(Math.round(minorToMajorTickSpacingRatio * (SLIDER_MAX - SLIDER_MIN)));
		slider.setMajorTickSpacing(Math.round(majorToMajorTickSpacingRatio * (SLIDER_MAX - SLIDER_MIN)));
		slider.setLabelTable(getLabelTable(model));
	}

	public Number getValue() {
		return (Number) spinner.getValue();
	}

	public void setValue(Object value) {
		spinner.setValue(value);
	}

	public float getMinorToMajorTickSpacingRatio() {
		return minorToMajorTickSpacingRatio;
	}

	public void setMinorToMajorTickSpacingRatio(float minorToMajorTickSpacingRatio) {
		this.minorToMajorTickSpacingRatio = minorToMajorTickSpacingRatio;
		setModel(getModel());
	}

	public float getMajorToMajorTickSpacingRatio() {
		return majorToMajorTickSpacingRatio;
	}

	public void setMajorToMajorTickSpacingRatio(float majorToMajorTickSpacingRatio) {
		this.majorToMajorTickSpacingRatio = majorToMajorTickSpacingRatio;
		setModel(getModel());
	}

	public void addChangeListener(ChangeListener listener) {
		spinner.addChangeListener(listener);
	}

	public void removeChangeListener(ChangeListener listener) {
		spinner.removeChangeListener(listener);
	}

	public ChangeListener[] getChangeListeners() {
		return spinner.getChangeListeners();
	}

	protected Dictionary<Integer, Component> getLabelTable(SpinnerNumberModel model) {
		Dictionary<Integer, Component> result = new Hashtable<Integer, Component>();
		int increment = slider.getMajorTickSpacing();
		for (int i = SLIDER_MIN; i <= SLIDER_MAX; i += increment) {
			result.put(i, new JLabel(formatLabelValue((Number) convertSliderToSpinnerValue(i))));
		}
		return result;
	}

	protected String formatLabelValue(Number value) {
		return NumberFormat.getInstance().format(value);
	}

	public boolean getSnapToTicks() {
		return slider.getSnapToTicks();
	}

	public void setSnapToTicks(boolean b) {
		slider.setSnapToTicks(b);
	}

	public boolean getPaintTicks() {
		return slider.getPaintTicks();
	}

	public void setPaintTicks(boolean b) {
		slider.setPaintTicks(b);
	}

	public boolean getPaintTrack() {
		return slider.getPaintTrack();
	}

	public void setPaintTrack(boolean b) {
		slider.setPaintTrack(b);
	}

	public boolean getPaintLabels() {
		return slider.getPaintLabels();
	}

	public void setPaintLabels(boolean b) {
		slider.setPaintLabels(b);
	}

}
