package com.otk.application.window.component.crud;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.otk.application.error.Assertion;
import com.otk.application.error.StandardError;
import com.otk.application.translation.T;
import com.otk.application.util.FileUtils;
import com.otk.application.util.MiscUtils;
import com.otk.application.util.TextUtils;

public abstract class AbstractCrudPanel extends JSplitPane
		implements ICrudItemControlListener, AbstractCrudModel.IListener {

	private static final long serialVersionUID = 1L;

	private JList<Object> listControl;
	private boolean listExplicitSelectionChange = true;
	private int lastListSelection = -1;
	private boolean listSelectionHandlingEnabled = true;
	private AbstractCrudItem current;
	private Component currentComponent;
	private JPanel fieldsPanelContainer;
	private JButton addButton;
	private JButton deleteButton;
	private JButton duplicateButton;
	private JButton clearButton;
	private JButton moveUpButton;
	private JButton moveDownButton;
	private JButton importButton;
	private JButton exportFButton;
	private AbstractCrudModel model;
	private JPanel buttonPanel;

	protected abstract AbstractCrudModel createModel();

	public AbstractCrudPanel(String listTitle) {
		JPanel listPanel = new JPanel();
		listPanel.setLayout(new BorderLayout(0, 0));

		JLabel listLabel = new JLabel(listTitle);
		listLabel.setHorizontalAlignment(SwingConstants.CENTER);
		listPanel.add(listLabel, BorderLayout.NORTH);

		listControl = createListControl();
		listControl.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				onSelectionChange();
			}
		});
		JScrollPane listControlScrolledContainer = new JScrollPane(listControl);
		listPanel.add(listControlScrolledContainer, BorderLayout.CENTER);

		JPanel buttonPanelContainer = new JPanel();
		listPanel.add(buttonPanelContainer, BorderLayout.WEST);
		buttonPanelContainer.setLayout(new BorderLayout());

		buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(0, 1));
		buttonPanelContainer.add(BorderLayout.NORTH, buttonPanel);

		addButton = new JButton(T.t("Add"));
		buttonPanel.add(addButton);
		addButton.setToolTipText(T.t("Add an item"));
		addButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addButtonPressed();
			}
		});

		deleteButton = new JButton(T.t("Delete"));
		buttonPanel.add(deleteButton);
		deleteButton.setToolTipText(T.t("Delete the item"));
		deleteButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				deleteButtonPressed();
			}
		});

		duplicateButton = new JButton(T.t("Duplicate"));
		buttonPanel.add(duplicateButton);
		duplicateButton.setToolTipText(T.t("Duplicate the item"));
		duplicateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				duplicateButtonPressed();
			}
		});

		clearButton = new JButton(T.t("Clear"));
		buttonPanel.add(clearButton);
		clearButton.setToolTipText(T.t("Delete all the items"));
		clearButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				clearButtonPressed();
			}
		});

		moveUpButton = new JButton(T.t("Move up"));
		buttonPanel.add(moveUpButton);
		moveUpButton.setToolTipText(T.t("Move the item up"));
		moveUpButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				moveUpButtonPressed();
			}

		});

		moveDownButton = new JButton(T.t("Move down"));
		buttonPanel.add(moveDownButton);
		moveDownButton.setToolTipText(T.t("Move the item down"));
		moveDownButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				moveDownButtonPressed();
			}
		});

		importButton = new JButton(T.t("Import..."));
		buttonPanel.add(importButton);
		importButton.setToolTipText(T.t("Import the list"));
		importButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				importButtonPressed();
			}
		});

		exportFButton = new JButton(T.t("Export..."));
		buttonPanel.add(exportFButton);
		importButton.setToolTipText(T.t("Export the list"));
		exportFButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exportButtonPressed();
			}
		});

		fieldsPanelContainer = new JPanel();
		fieldsPanelContainer.setLayout(new BorderLayout());
		fieldsPanelContainer.setPreferredSize(new Dimension(100, 100));
		fieldsPanelContainer.setMinimumSize(new Dimension(0, 0));

		setOrientation(JSplitPane.HORIZONTAL_SPLIT);
		setLeftComponent(listPanel);
		listPanel.setMinimumSize(new Dimension(0, 0));
		setRightComponent(fieldsPanelContainer);
		setMinimumSize(new Dimension(10, 10));
		setResizeWeight(0.5);
		setDividerLocation(0.5);
	}

	public void connectToModel() {
		getModel();
	}

	public Icon getIcon(AbstractCrudItem item) {
		return null;
	}

	public String getLabelText(AbstractCrudItem item) {
		return getModel().serialize(item);
	}

	public AbstractCrudItem getCurrentCrudItem() {
		return current;
	}

	@Override
	public void added(int index) {
		listControl.setSelectedIndex(index);
		listControl.ensureIndexIsVisible(index);
		onSelectionChange();
		onListStateChange();
	}

	public AbstractCrudModel getModel() {
		if (model == null) {
			model = createModel();
			model.getListeners().add(this);
			model.conectTo(listControl);
		}
		return model;
	}

	public JPanel getButtonPanel() {
		return buttonPanel;
	}

	public JButton getAddButton() {
		return addButton;
	}

	public JButton getDeleteButton() {
		return deleteButton;
	}

	public JButton getDuplicateButton() {
		return duplicateButton;
	}

	public JButton getClearButton() {
		return clearButton;
	}

	public JButton getMoveUpButton() {
		return moveUpButton;
	}

	public JButton getMoveDownButton() {
		return moveDownButton;
	}

	public JButton getImportButton() {
		return importButton;
	}

	public JButton getExportButton() {
		return exportFButton;
	}

	public boolean userFinalizedPendingModications(AbstractCrudItem current) {
		int response = JOptionPane.showConfirmDialog(listControl,
				T.t("You did not apply the last changes." + "\nApply now?"
						+ "\n(Otherwise these changes will be lost)"),
				T.t("Pending Modifications"), JOptionPane.YES_NO_CANCEL_OPTION);
		if (response == JOptionPane.YES_OPTION) {
			current.onApplyRequested(this);
			return true;
		} else if (response == JOptionPane.NO_OPTION) {
			return true;
		} else {
			return false;
		}
	}

	protected Class<? extends AbstractCrudItem> askType() {
		List<Object> options = new ArrayList<Object>();
		for (Class<? extends AbstractCrudItem> t : getModel().getItemTypes()) {
			options.add(getModel().getItemTypeCaption(t));
		}
		String option = (String) JOptionPane.showInputDialog(AbstractCrudPanel.this, T.t("Choose the item type:"),
				T.t("Type Selection"), JOptionPane.QUESTION_MESSAGE, null, options.toArray(), options.get(0));
		if (option == null) {
			return null;
		}
		int index = options.indexOf(option);
		return getModel().getItemTypes()[index];
	}

	protected void addButtonPressed() {
		if ((current != null) && current.isModifying()) {
			if (!userFinalizedPendingModications(current)) {
				return;
			}
		}
		Class<? extends AbstractCrudItem> itemType;
		if (getModel().getItemTypes().length == 1) {
			itemType = getModel().getItemTypes()[0];
		} else {
			itemType = askType();
			if (itemType == null) {
				return;
			}
		}
		beforeListStateChangeAction();
		AbstractCrudItem newItem = getModel().createItem(itemType);
		listExplicitSelectionChange = false;
		if (listControl.getSelectedIndex() == -1) {
			getModel().add(listControl.getSelectedIndex(), newItem);
		} else {
			getModel().add(listControl.getSelectedIndex() + 1, newItem);
		}
		listExplicitSelectionChange = true;
	}

	protected void clearButtonPressed() {
		if (JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog(AbstractCrudPanel.this,
				T.t("Do you really want to delete all the items?"), T.t("Confirmation"),
				JOptionPane.OK_CANCEL_OPTION)) {
			return;
		}
		listExplicitSelectionChange = false;
		getModel().clear();
		listExplicitSelectionChange = true;
	}

	@Override
	public void cleared() {
		onSelectionChange();
		onListStateChange();
	}

	protected void deleteButtonPressed() {
		if (listControl.getSelectedIndex() < 0) {
			return;
		}
		if (JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog(AbstractCrudPanel.this,
				T.t("Do you really want to delete the item?"), T.t("Confirmation"), JOptionPane.OK_CANCEL_OPTION)) {
			return;
		}
		beforeListStateChangeAction();
		listExplicitSelectionChange = false;
		getModel().delete(listControl.getSelectedIndex());
		listExplicitSelectionChange = true;
	}

	@Override
	public void deleted(int index) {
		listControl.setSelectedIndex(index - 1);
		onSelectionChange();
		onListStateChange();
	}

	protected void exportButtonPressed() {
		String filePath = FileUtils.askFilePath(this, T.t("Save"), new String[] { getFileNameExtension() }, null);
		if (filePath == null) {
			return;
		}
		String modelString = getModel().save();
		try {
			FileUtils.write(new File(filePath), modelString, false);
		} catch (Exception e) {
			throw new StandardError(e.toString(), e);
		}
	}

	protected String getFileNameExtension() {
		return "txt";
	}

	protected void importButtonPressed() {
		String filePath = FileUtils.askFilePath(this, T.t("Open"), new String[] { getFileNameExtension() }, null);
		if (filePath == null) {
			return;
		}
		String modelString;
		try {
			modelString = FileUtils.read(new File(filePath));
		} catch (Exception e) {
			throw new StandardError(e.toString(), e);
		}
		beforeListStateChangeAction();
		listExplicitSelectionChange = false;
		getModel().load(modelString, true, listControl.getSelectedIndex());
		listExplicitSelectionChange = true;
	}

	@Override
	public void batchChange() {
		onListStateChange();
		onSelectionChange();
	}

	public AbstractCrudItem getSelected() {
		if (listControl.getSelectedIndex() < 0) {
			return null;
		}
		return (AbstractCrudItem) getModel().getList().get(listControl.getSelectedIndex());
	}

	public void setSelected(AbstractCrudItem element) {
		if (element == null) {
			listControl.clearSelection();
		} else {
			final int index = getModel().getList().indexOf(element);
			Assertion.check(new Assertion.Predicate() {
				@Override
				protected boolean isTrue() {
					return index >= 0;
				}
			});
			listControl.setSelectedIndex(index);
		}
	}

	protected void duplicateButtonPressed() {
		if (listControl.getSelectedIndex() < 0) {
			return;
		}
		if ((current != null) && current.isModifying()) {
			if (!userFinalizedPendingModications(current)) {
				return;
			}
		}
		beforeListStateChangeAction();
		listExplicitSelectionChange = false;
		getModel().duplicate(listControl.getSelectedIndex());
		listExplicitSelectionChange = true;
	}

	@Override
	public void duplicated(int i) {
		listControl.setSelectedIndex(i);
		onSelectionChange();
		onListStateChange();
	}

	protected void moveDownButtonPressed() {
		if (listControl.getSelectedIndex() < 0) {
			return;
		}
		if (listControl.getSelectedIndex() == (getModel().getList().size() - 1)) {
			return;
		}
		beforeListStateChangeAction();
		listExplicitSelectionChange = false;
		getModel().moveDown(listControl.getSelectedIndex());
		listExplicitSelectionChange = true;
	}

	@Override
	public void movedDown(int i) {
		listControl.setSelectedIndex(i);
		onListStateChange();
	}

	protected void moveUpButtonPressed() {
		if (listControl.getSelectedIndex() <= 0) {
			return;
		}
		beforeListStateChangeAction();
		listExplicitSelectionChange = false;
		getModel().moveUp(listControl.getSelectedIndex());
		listExplicitSelectionChange = true;
	}

	@Override
	public void movedUp(int i) {
		listControl.setSelectedIndex(i);
		onListStateChange();
	}

	@Override
	public void beforeItemUpdate(AbstractCrudItem item) {
		beforeListStateChangeAction();
	}

	@Override
	public void itemUpdated(AbstractCrudItem item) {
		getModel().notifyUpdate(item);
		onListStateChange();
	}

	@Override
	public void updated(int index) {
		onListStateChange();
	}

	protected void onSelectionChange() {
		if (!listSelectionHandlingEnabled) {
			return;
		}
		if (listExplicitSelectionChange) {
			if (listControl.getSelectedIndex() != lastListSelection) {
				if ((current != null) && current.isModifying()) {
					if (!userFinalizedPendingModications(current)) {
						listSelectionHandlingEnabled = false;
						listControl.setSelectedIndex(lastListSelection);
						listSelectionHandlingEnabled = true;
						return;
					}
				}
			}
		}

		current = null;
		if (listControl.getSelectedIndex() >= 0) {
			current = (AbstractCrudItem) getModel().getList().get(listControl.getSelectedIndex());
		}
		if (currentComponent != null) {
			fieldsPanelContainer.remove(currentComponent);
			currentComponent = null;
			fieldsPanelContainer.repaint();
		}
		if (current != null) {
			currentComponent = current.createComponent(this);
			fieldsPanelContainer.add(currentComponent);
		}
		fieldsPanelContainer.validate();
		lastListSelection = listControl.getSelectedIndex();
	}

	protected abstract void beforeListStateChangeAction();

	protected abstract void onListStateChange();

	protected JList<Object> createListControl() {
		JList<Object> result = new JList<Object>();
		result.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		result.setCellRenderer(new DefaultListCellRenderer() {

			private static final long serialVersionUID = 1L;

			@Override
			public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				JLabel label = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected,
						cellHasFocus);
				AbstractCrudItem item = (AbstractCrudItem) value;

				label.setText(AbstractCrudPanel.this.getLabelText(item));
				label.setIcon(AbstractCrudPanel.this.getIcon(item));
				label.setText(TextUtils.truncateNicely(label.getText(), 100));

				label.setText(TextUtils.escapeHTML(label.getText()));

				Color labelBackground;
				Color labelForeground;
				if (isSelected) {
					labelBackground = UIManager.getColor("Tree.selectionBackground");
					labelForeground = UIManager.getColor("Tree.selectionForeground");
				} else {
					if ((index % 2) == 0) {
						labelBackground = UIManager.getColor("Table.background");
						labelForeground = UIManager.getColor("Table.foreground");
					} else {
						labelBackground = UIManager.getColor("TableHeader.background");
						labelForeground = UIManager.getColor("TableHeader.foreground");
					}
				}
				label.setBackground(labelBackground);
				label.setText(
						"<font color=\"" + MiscUtils.toHTML(labelForeground) + "\">" + label.getText() + "</font>");

				AbstractCrudPanel.this.decorateItemEntryLabel(list, item, index, isSelected, cellHasFocus, label);
				label.setText("<html>" + label.getText() + "</html>");

				return label;
			}
		});
		return result;
	}

	protected void decorateItemEntryLabel(JList<?> list, AbstractCrudItem item, int index, boolean isSelected,
			boolean cellHasFocus, JLabel label) {
	}

}