package com.otk.application.window.component.crud;

import java.util.ArrayList;
import java.util.List;

import com.otk.application.error.Assertion;
import com.otk.application.error.UnexpectedError;

public class SingleTypeCrudPanel<T> extends AbstractCrudPanel {

	private static final long serialVersionUID = 1L;

	private Class<? extends AbstractPojoWrapperCrudItem<T>> itemWrapperClass;

	public SingleTypeCrudPanel(Class<? extends AbstractPojoWrapperCrudItem<T>> itemWrapperClass, String listTitle) {
		super(listTitle);
		this.itemWrapperClass = itemWrapperClass;
	}

	@Override
	protected SingleTypeCrudModel<T> createModel() {
		return new SingleTypeCrudModel<T>(itemWrapperClass);
	}

	public Class<? extends AbstractPojoWrapperCrudItem<T>> getItemWrapperClass() {
		return itemWrapperClass;
	}

	@Override
	public AbstractCrudModel getModel() {
		final AbstractCrudModel result = super.getModel();
		Assertion.check(new Assertion.Predicate() {
			@Override
			protected void doesNotFail() throws Throwable {
				if (!(result instanceof SingleTypeCrudModel)) {
					throw new Exception();
				}
				@SuppressWarnings("unchecked")
				SingleTypeCrudModel<T> model = (SingleTypeCrudModel<T>) result;
				if (model.itemWrapperClass != itemWrapperClass) {
					throw new Exception();
				}
				AbstractPojoWrapperCrudItem<T> item = model.createItem();
				if (item.getClass() != model.itemWrapperClass) {
					throw new Exception();
				}
			}
		});
		return result;
	}

	@Override
	protected void beforeListStateChangeAction() {
		// nothing
	}

	@Override
	protected void onListStateChange() {
		// nothing
	}

	public static class SingleTypeCrudModel<T> extends AbstractCrudModel {

		private Class<? extends AbstractPojoWrapperCrudItem<T>> itemWrapperClass;

		public SingleTypeCrudModel(Class<? extends AbstractPojoWrapperCrudItem<T>> itemWrapperClass) {
			super();
			this.itemWrapperClass = itemWrapperClass;
		}

		public List<T> getUnwrappedList() {
			List<T> result = new ArrayList<T>();
			for (AbstractCrudItem item : getList()) {
				@SuppressWarnings("unchecked")
				AbstractPojoWrapperCrudItem<T> wrapper = (AbstractPojoWrapperCrudItem<T>) item;
				result.add(wrapper.getWrapped());
			}
			return result;
		}

		public void setUnwrappedList(List<T> unwrappeds) {
			List<AbstractCrudItem> wrappers = new ArrayList<AbstractCrudItem>();
			for (T unwrapped : unwrappeds) {
				AbstractPojoWrapperCrudItem<T> wrapper = createItem();
				wrapper.setWrapped(unwrapped);
				wrappers.add(wrapper);
			}
			setList(wrappers);
		}

		@SuppressWarnings("unchecked")
		@Override
		protected Class<? extends AbstractCrudItem>[] getItemTypes() {
			return new Class[] { itemWrapperClass };
		}

		@SuppressWarnings("unchecked")
		public Class<T> getWrappedItemType() {
			try {
				AbstractPojoWrapperCrudItem<T> wrapper = createItem();
				T wrapped = wrapper.createWrapped();
				return (Class<T>) wrapped.getClass();
			} catch (Exception e) {
				throw new UnexpectedError(e);
			}
		}

		@Override
		public String getItemName(final Class<? extends AbstractCrudItem> c) {
			Assertion.check(new Assertion.Predicate() {
				@Override
				protected boolean isTrue() {
					return c == itemWrapperClass;
				}
			});
			return getWrappedItemType().getSimpleName();
		}

		@SuppressWarnings("unchecked")
		@Override
		final protected <U extends AbstractCrudItem> U createItem(Class<U> itemType) {
			return (U) createItem();
		}

		protected AbstractPojoWrapperCrudItem<T> createItem() {
			return super.createItem(itemWrapperClass);
		}

		@Override
		public Class<? extends AbstractCrudItem> getItemType(final String typeName) {
			Assertion.check(new Assertion.Predicate() {
				@Override
				protected boolean isTrue() {
					return getWrappedItemType().getSimpleName().equals(typeName);
				}
			});
			return getItemType();
		}

		public Class<? extends AbstractCrudItem> getItemType() {
			return itemWrapperClass;
		}

	}

}
